@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -31,7 +29,7 @@ ScriptConfigItem _start_date_config = {
 	AI::START_NEXT_DEVIATION,
 	30,
 	SCRIPTCONFIG_NONE,
-	NULL,
+	nullptr,
 	false
 };
 
@@ -52,7 +50,7 @@ AIConfig::AIConfig(const AIConfig *config) : ScriptConfig(config)
 	} else {
 		config = &_settings_game.ai_config[company];
 	}
-	if (*config == NULL) *config = new AIConfig();
+	if (*config == nullptr) *config = new AIConfig();
 	return *config;
 }
 
@@ -69,7 +67,7 @@ ScriptInfo *AIConfig::FindInfo(const char *name, int version, bool force_exact_m
 bool AIConfig::ResetInfo(bool force_exact_match)
 {
 	this->info = (ScriptInfo *)AI::FindInfo(this->name, force_exact_match ? this->version : -1, force_exact_match);
-	return this->info != NULL;
+	return this->info != nullptr;
 }
 
 void AIConfig::PushExtraConfigList()
@@ -90,7 +88,7 @@ void AIConfig::ClearConfigList()
 
 int AIConfig::GetSetting(const char *name) const
 {
-	if (this->info == NULL) {
+	if (this->info == nullptr) {
 		SettingValueList::const_iterator it = this->settings.find(name);
 		if (it == this->settings.end()) {
 			assert(strcmp("start_date", name) == 0);
@@ -111,7 +109,7 @@ int AIConfig::GetSetting(const char *name) const
 
 void AIConfig::SetSetting(const char *name, int value)
 {
-	if (this->info == NULL) {
+	if (this->info == nullptr) {
 		if (strcmp("start_date", name) != 0) return;
 		value = Clamp(value, AI::START_NEXT_MIN, AI::START_NEXT_MAX);
 
@@ -127,3 +125,14 @@ void AIConfig::SetSetting(const char *name, int value)
 
 	ScriptConfig::SetSetting(name, value);
 }
+
+void AIConfig::AddRandomDeviation()
+{
+	int start_date = this->GetSetting("start_date");
+
+	ScriptConfig::AddRandomDeviation();
+
+	/* start_date = 0 is a special case, where random deviation does not occur.
+	 * If start_date was not already 0, then a minimum value of 1 must apply. */
+	this->SetSetting("start_date", start_date != 0 ? max(1, this->GetSetting("start_date")) : 0);
+}
