@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -76,9 +74,9 @@ struct BaseSet {
 	{
 		free(this->name);
 
-		for (TranslatedStrings::iterator iter = this->description.Begin(); iter != this->description.End(); iter++) {
-			free(iter->first);
-			free(iter->second);
+		for (auto &pair : this->description) {
+			free(pair.first);
+			free(pair.second);
 		}
 
 		for (uint i = 0; i < NUM_FILES; i++) {
@@ -118,20 +116,20 @@ struct BaseSet {
 	 * @param isocode the isocode to search for
 	 * @return the description
 	 */
-	const char *GetDescription(const char *isocode = NULL) const
+	const char *GetDescription(const char *isocode = nullptr) const
 	{
-		if (isocode != NULL) {
+		if (isocode != nullptr) {
 			/* First the full ISO code */
-			for (TranslatedStrings::const_iterator iter = this->description.Begin(); iter != this->description.End(); iter++) {
-				if (strcmp(iter->first, isocode) == 0) return iter->second;
+			for (const auto &pair : this->description) {
+				if (strcmp(pair.first, isocode) == 0) return pair.second;
 			}
 			/* Then the first two characters */
-			for (TranslatedStrings::const_iterator iter = this->description.Begin(); iter != this->description.End(); iter++) {
-				if (strncmp(iter->first, isocode, 2) == 0) return iter->second;
+			for (const auto &pair : this->description) {
+				if (strncmp(pair.first, isocode, 2) == 0) return pair.second;
 			}
 		}
 		/* Then fall back */
-		return this->description.Begin()->second;
+		return this->description.front().second;
 	}
 
 	/**
@@ -151,17 +149,17 @@ struct BaseSet {
 	/**
 	 * Search a textfile file next to this base media.
 	 * @param type The type of the textfile to search for.
-	 * @return The filename for the textfile, \c NULL otherwise.
+	 * @return The filename for the textfile, \c nullptr otherwise.
 	 */
 	const char *GetTextfile(TextfileType type) const
 	{
 		for (uint i = 0; i < NUM_FILES; i++) {
 			const char *textfile = ::GetTextfile(type, BASESET_DIR, this->files[i].filename);
-			if (textfile != NULL) {
+			if (textfile != nullptr) {
 				return textfile;
 			}
 		}
-		return NULL;
+		return nullptr;
 	}
 };
 
@@ -176,7 +174,7 @@ protected:
 	static Tbase_set *duplicate_sets; ///< All sets that aren't available, but needed for not downloading base sets when a newer version than the one on BaNaNaS is loaded.
 	static const Tbase_set *used_set; ///< The currently used set
 
-	/* virtual */ bool AddFile(const char *filename, size_t basepath_length, const char *tar_filename);
+	bool AddFile(const char *filename, size_t basepath_length, const char *tar_filename) override;
 
 	/**
 	 * Get the extension that is used to identify this set.
@@ -231,7 +229,7 @@ template <class Tbase_set> /* static */ Tbase_set *BaseMedia<Tbase_set>::duplica
  * @param ci The content info to compare it to.
  * @param md5sum Should the MD5 checksum be tested as well?
  * @param s The list with sets.
- * @return The filename of the first file of the base set, or \c NULL if there is no match.
+ * @return The filename of the first file of the base set, or \c nullptr if there is no match.
  */
 template <class Tbase_set>
 const char *TryGetBaseSetFile(const ContentInfo *ci, bool md5sum, const Tbase_set *s);
