@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -30,7 +28,7 @@ void IConsoleWarning(const char *string);
 void IConsoleError(const char *string);
 
 /* Parser */
-void IConsoleCmdExec(const char *cmdstr);
+void IConsoleCmdExec(const char *cmdstr, const uint recurse_count = 0);
 
 bool IsValidConsoleColour(TextColour c);
 
