@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -23,10 +21,8 @@
 
 #include <time.h>
 
-#if defined(ENABLE_NETWORK)
 #include "network/network_admin.h"
 SOCKET _debug_socket = INVALID_SOCKET;
-#endif /* ENABLE_NETWORK */
 
 #include "safeguards.h"
 
@@ -111,7 +107,6 @@ char *DumpDebugFacilityNames(char *buf, char *last)
  */
 static void debug_print(const char *dbg, const char *buf)
 {
-#if defined(ENABLE_NETWORK)
 	if (_debug_socket != INVALID_SOCKET) {
 		char buf2[1024 + 32];
 
@@ -121,17 +116,16 @@ static void debug_print(const char *dbg, const char *buf)
 		send(_debug_socket, buf2, (int)strlen(buf2), 0);
 		return;
 	}
-#endif /* ENABLE_NETWORK */
 	if (strcmp(dbg, "desync") == 0) {
 		static FILE *f = FioFOpenFile("commands-out.log", "wb", AUTOSAVE_DIR);
-		if (f == NULL) return;
+		if (f == nullptr) return;
 
 		fprintf(f, "%s%s\n", GetLogPrefix(), buf);
 		fflush(f);
 #ifdef RANDOM_DEBUG
 	} else if (strcmp(dbg, "random") == 0) {
 		static FILE *f = FioFOpenFile("random-out.log", "wb", AUTOSAVE_DIR);
-		if (f == NULL) return;
+		if (f == nullptr) return;
 
 		fprintf(f, "%s\n", buf);
 		fflush(f);
@@ -146,9 +140,7 @@ static void debug_print(const char *dbg, const char *buf)
 #else
 		fputs(buffer, stderr);
 #endif
-#ifdef ENABLE_NETWORK
 		NetworkAdminConsole(dbg, buf);
-#endif /* ENABLE_NETWORK */
 		IConsoleDebug(dbg, buf);
 	}
 }
@@ -206,7 +198,7 @@ void SetDebugString(const char *s)
 		while (*s >= 'a' && *s <= 'z') s++;
 
 		/* check debugging levels */
-		p = NULL;
+		p = nullptr;
 		for (i = debug_level; i != endof(debug_level); ++i) {
 			if (s == t + strlen(i->name) && strncmp(t, i->name, s - t) == 0) {
 				p = i->level;
@@ -217,7 +209,7 @@ void SetDebugString(const char *s)
 		if (*s == '=') s++;
 		v = strtoul(s, &end, 0);
 		s = end;
-		if (p != NULL) {
+		if (p != nullptr) {
 			*p = v;
 		} else {
 			ShowInfoF("Unknown debug level '%.*s'", (int)(s - t), t);
@@ -252,13 +244,13 @@ const char *GetDebugString()
 /**
  * Get the prefix for logs; if show_date_in_logs is enabled it returns
  * the date, otherwise it returns nothing.
- * @return the prefix for logs (do not free), never NULL
+ * @return the prefix for logs (do not free), never nullptr
  */
 const char *GetLogPrefix()
 {
 	static char _log_prefix[24];
 	if (_settings_client.gui.show_date_in_logs) {
-		time_t cur_time = time(NULL);
+		time_t cur_time = time(nullptr);
 		strftime(_log_prefix, sizeof(_log_prefix), "[%Y-%m-%d %H:%M:%S] ", localtime(&cur_time));
 	} else {
 		*_log_prefix = '\0';
