@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -13,6 +11,7 @@
 #define DEBUG_H
 
 #include "cpu.h"
+#include <chrono>
 
 /* Debugging messages policy:
  * These should be the severities used for direct DEBUG() calls
@@ -83,21 +82,40 @@ const char *GetDebugString();
  *
  * TIC() / TOC() creates its own block, so make sure not the mangle
  *  it with another block.
+ *
+ * The output is counted in CPU cycles, and not comparable across
+ *  machines. Mainly useful for local optimisations.
  **/
 #define TIC() {\
 	uint64 _xxx_ = ottd_rdtsc();\
-	static uint64 __sum__ = 0;\
-	static uint32 __i__ = 0;
+	static uint64 _sum_ = 0;\
+	static uint32 _i_ = 0;
 
 #define TOC(str, count)\
-	__sum__ += ottd_rdtsc() - _xxx_;\
-	if (++__i__ == count) {\
-		DEBUG(misc, 0, "[%s] " OTTD_PRINTF64 " [avg: %.1f]", str, __sum__, __sum__/(double)__i__);\
-		__i__ = 0;\
-		__sum__ = 0;\
+	_sum_ += ottd_rdtsc() - _xxx_;\
+	if (++_i_ == count) {\
+		DEBUG(misc, 0, "[%s] " OTTD_PRINTF64 " [avg: %.1f]", str, _sum_, _sum_/(double)_i_);\
+		_i_ = 0;\
+		_sum_ = 0;\
 	}\
 }
 
+/* Chrono based version. The output is in microseconds. */
+#define TICC() {\
+	auto _start_ = std::chrono::high_resolution_clock::now();\
+	static uint64 _sum_ = 0;\
+	static uint32 _i_ = 0;
+
+#define TOCC(str, _count_)\
+	_sum_ += (std::chrono::duration_cast<std::chrono::microseconds>(std::chrono::high_resolution_clock::now() - _start_)).count();\
+	if (++_i_ == _count_) {\
+		DEBUG(misc, 0, "[%s] " OTTD_PRINTF64 " us [avg: %.1f us]", str, _sum_, _sum_/(double)_i_);\
+		_i_ = 0;\
+		_sum_ = 0;\
+	}\
+}
+
+
 void ShowInfo(const char *str);
 void CDECL ShowInfoF(const char *str, ...) WARN_FORMAT(1, 2);
 
