@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -147,7 +145,7 @@ typedef Money Prices[PR_END]; ///< Prices of everything. @see Price
 typedef int8 PriceMultipliers[PR_END];
 
 /** Types of expenses. */
-enum ExpensesType {
+enum ExpensesType : byte {
 	EXPENSES_CONSTRUCTION =  0,   ///< Construction costs.
 	EXPENSES_NEW_VEHICLES,        ///< New vehicles.
 	EXPENSES_TRAIN_RUN,           ///< Running costs trains.
@@ -167,7 +165,6 @@ enum ExpensesType {
 
 /** Define basic enum properties for ExpensesType */
 template <> struct EnumPropsT<ExpensesType> : MakeEnumPropsT<ExpensesType, byte, EXPENSES_CONSTRUCTION, EXPENSES_END, INVALID_EXPENSES, 8> {};
-typedef TinyEnumT<ExpensesType> ExpensesTypeByte; ///< typedefing-enumification of ExpensesType
 
 /**
  * Categories of a price bases.
