@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -34,7 +32,7 @@ struct Engine : EnginePool::PoolItem<&_engine_pool> {
 	uint16 duration_phase_3;    ///< Third reliability phase on months, decaying to #reliability_final.
 	byte flags;                 ///< Flags of the engine. @see EngineFlags
 	CompanyMask preview_asked;  ///< Bit for each company which has already been offered a preview.
-	CompanyByte preview_company;///< Company which is currently being offered a preview \c INVALID_COMPANY means no company.
+	CompanyID preview_company;  ///< Company which is currently being offered a preview \c INVALID_COMPANY means no company.
 	byte preview_wait;          ///< Daily countdown timer for timeout of offering the engine to the #preview_company company.
 	CompanyMask company_avail;  ///< Bit for each company whether the engine is available for that company.
 	CompanyMask company_hidden; ///< Bit for each company whether the engine is normally hidden in the build gui for that company.
@@ -83,7 +81,7 @@ struct Engine : EnginePool::PoolItem<&_engine_pool> {
 		return this->info.cargo_type;
 	}
 
-	uint DetermineCapacity(const Vehicle *v, uint16 *mail_capacity = NULL) const;
+	uint DetermineCapacity(const Vehicle *v, uint16 *mail_capacity = nullptr) const;
 
 	bool CanCarryCargo() const;
 
@@ -98,9 +96,9 @@ struct Engine : EnginePool::PoolItem<&_engine_pool> {
 	 * @return The default capacity
 	 * @see GetDefaultCargoType
 	 */
-	uint GetDisplayDefaultCapacity(uint16 *mail_capacity = NULL) const
+	uint GetDisplayDefaultCapacity(uint16 *mail_capacity = nullptr) const
 	{
-		return this->DetermineCapacity(NULL, mail_capacity);
+		return this->DetermineCapacity(nullptr, mail_capacity);
 	}
 
 	Money GetRunningCost() const;
@@ -118,7 +116,7 @@ struct Engine : EnginePool::PoolItem<&_engine_pool> {
 	 * @param c Company to check.
 	 * @return \c true iff the engine is hidden in the GUI for the given company.
 	 */
-	inline bool IsHidden(CompanyByte c) const
+	inline bool IsHidden(CompanyID c) const
 	{
 		return c < MAX_COMPANIES && HasBit(this->company_hidden, c);
 	}
@@ -143,12 +141,29 @@ struct Engine : EnginePool::PoolItem<&_engine_pool> {
 	}
 
 	uint32 GetGRFID() const;
+
+	struct EngineTypeFilter {
+		VehicleType vt;
+
+		bool operator() (size_t index) { return Engine::Get(index)->type == this->vt; }
+	};
+
+	/**
+	 * Returns an iterable ensemble of all valid engines of the given type
+	 * @param vt the VehicleType for engines to be valid
+	 * @param from index of the first engine to consider
+	 * @return an iterable ensemble of all valid engines of the given type
+	 */
+	static Pool::IterateWrapperFiltered<Engine, EngineTypeFilter> IterateType(VehicleType vt, size_t from = 0)
+	{
+		return Pool::IterateWrapperFiltered<Engine, EngineTypeFilter>(from, EngineTypeFilter{ vt });
+	}
 };
 
 struct EngineIDMapping {
 	uint32 grfid;          ///< The GRF ID of the file the entity belongs to
 	uint16 internal_id;    ///< The internal ID within the GRF file
-	VehicleTypeByte type;  ///< The engine type
+	VehicleType type;      ///< The engine type
 	uint8  substitute_id;  ///< The (original) entity ID to use if this GRF is not available (currently not used)
 };
 
@@ -156,7 +171,7 @@ struct EngineIDMapping {
  * Stores the mapping of EngineID to the internal id of newgrfs.
  * Note: This is not part of Engine, as the data in the EngineOverrideManager and the engine pool get resetted in different cases.
  */
-struct EngineOverrideManager : SmallVector<EngineIDMapping, 256> {
+struct EngineOverrideManager : std::vector<EngineIDMapping> {
 	static const uint NUM_DEFAULT_ENGINES; ///< Number of default entries
 
 	void ResetToDefaultMapping();
@@ -167,11 +182,6 @@ struct EngineOverrideManager : SmallVector<EngineIDMapping, 256> {
 
 extern EngineOverrideManager _engine_mngr;
 
-#define FOR_ALL_ENGINES_FROM(var, start) FOR_ALL_ITEMS_FROM(Engine, engine_index, var, start)
-#define FOR_ALL_ENGINES(var) FOR_ALL_ENGINES_FROM(var, 0)
-
-#define FOR_ALL_ENGINES_OF_TYPE(e, engine_type) FOR_ALL_ENGINES(e) if (e->type == engine_type)
-
 static inline const EngineInfo *EngInfo(EngineID e)
 {
 	return &Engine::Get(e)->info;
