@@ -1,5 +1,3 @@
-/* $Id: fileio_func.h 26489 2014-04-23 21:23:21Z rubidium $ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -30,7 +28,7 @@ void FioSkipBytes(int n);
 /**
  * The search paths OpenTTD could search through.
  * At least one of the slots has to be filled with a path.
- * NULL paths tell that there is no such path for the
+ * nullptr paths tell that there is no such path for the
  * current operating system.
  */
 extern const char *_searchpaths[NUM_SEARCHPATHS];
@@ -42,19 +40,20 @@ extern const char *_searchpaths[NUM_SEARCHPATHS];
  */
 static inline bool IsValidSearchPath(Searchpath sp)
 {
-	return sp < NUM_SEARCHPATHS && _searchpaths[sp] != NULL;
+	return sp < NUM_SEARCHPATHS && _searchpaths[sp] != nullptr;
 }
 
 /** Iterator for all the search paths */
 #define FOR_ALL_SEARCHPATHS(sp) for (sp = SP_FIRST_DIR; sp < NUM_SEARCHPATHS; sp++) if (IsValidSearchPath(sp))
 
 void FioFCloseFile(FILE *f);
-FILE *FioFOpenFile(const char *filename, const char *mode, Subdirectory subdir, size_t *filesize = NULL);
+FILE *FioFOpenFile(const char *filename, const char *mode, Subdirectory subdir, size_t *filesize = nullptr);
 bool FioCheckFileExists(const char *filename, Subdirectory subdir);
 char *FioGetFullPath(char *buf, const char *last, Searchpath sp, Subdirectory subdir, const char *filename);
 char *FioFindFullPath(char *buf, const char *last, Subdirectory subdir, const char *filename);
 char *FioAppendDirectory(char *buf, const char *last, Searchpath sp, Subdirectory subdir);
 char *FioGetDirectory(char *buf, const char *last, Subdirectory subdir);
+void FioCreateDirectory(const char *name);
 
 const char *FiosGetScreenshotDir();
 
@@ -63,8 +62,6 @@ bool AppendPathSeparator(char *buf, const char *last);
 void DeterminePaths(const char *exe);
 void *ReadFileToMem(const char *filename, size_t *lenp, size_t maxsize);
 bool FileExists(const char *filename);
-const char *FioTarFirstDir(const char *tarname, Subdirectory subdir);
-void FioTarAddLink(const char *src, const char *dest, Subdirectory subdir);
 bool ExtractTar(const char *tar_filename, Subdirectory subdir);
 
 extern const char *_personal_dir; ///< custom directory for personal settings, saves, newgrf, etc.
@@ -106,7 +103,7 @@ public:
 		ALL      = BASESET | NEWGRF | AI | SCENARIO | GAME, ///< Scan for everything.
 	};
 
-	/* virtual */ bool AddFile(const char *filename, size_t basepath_length, const char *tar_filename = NULL);
+	bool AddFile(const char *filename, size_t basepath_length, const char *tar_filename = nullptr) override;
 
 	bool AddFile(Subdirectory sd, const char *filename);
 
@@ -117,7 +114,7 @@ public:
 DECLARE_ENUM_AS_BIT_SET(TarScanner::Mode)
 
 /* Implementation of opendir/readdir/closedir for Windows */
-#if defined(WIN32)
+#if defined(_WIN32)
 struct DIR;
 
 struct dirent { // XXX - only d_name implemented
@@ -135,7 +132,7 @@ int closedir(DIR *d);
 /* Use system-supplied opendir/readdir/closedir functions */
 # include <sys/types.h>
 # include <dirent.h>
-#endif /* defined(WIN32) */
+#endif /* defined(_WIN32) */
 
 /**
  * A wrapper around opendir() which will convert the string from
@@ -149,4 +146,17 @@ static inline DIR *ttd_opendir(const char *path)
 	return opendir(OTTD2FS(path));
 }
 
+
+/** Auto-close a file upon scope exit. */
+class FileCloser {
+	FILE *f;
+
+public:
+	FileCloser(FILE *_f) : f(_f) {}
+	~FileCloser()
+	{
+		fclose(f);
+	}
+};
+
 #endif /* FILEIO_FUNC_H */
