@@ -1,5 +1,3 @@
-/* $Id: fios.h 27729 2017-01-14 13:12:49Z frosch $ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -18,6 +16,13 @@
 #include "network/core/tcp_content.h"
 
 
+/** Special values for save-load window for the data parameter of #InvalidateWindowData. */
+enum SaveLoadInvalidateWindowData {
+	SLIWD_RESCAN_FILES,          ///< Rescan all files (when changed directory, ...)
+	SLIWD_SELECTION_CHANGES,     ///< File selection has changed (user click, ...)
+	SLIWD_FILTER_CHANGES,        ///< The filename filter has changed (via the editbox)
+};
+
 typedef SmallMap<uint, CompanyProperties *> CompanyPropertiesMap;
 
 /**
@@ -41,8 +46,8 @@ struct LoadCheckData {
 	struct LoggedAction *gamelog_action;          ///< Gamelog actions
 	uint gamelog_actions;                         ///< Number of gamelog actions
 
-	LoadCheckData() : error_data(NULL), grfconfig(NULL),
-			grf_compatibility(GLC_NOT_FOUND), gamelog_action(NULL), gamelog_actions(0)
+	LoadCheckData() : error_data(nullptr), grfconfig(nullptr),
+			grf_compatibility(GLC_NOT_FOUND), gamelog_action(nullptr), gamelog_actions(0)
 	{
 		this->Clear();
 	}
@@ -70,7 +75,7 @@ struct LoadCheckData {
 	 */
 	bool HasNewGrfs()
 	{
-		return this->checkable && this->error == INVALID_STRING_ID && this->grfconfig != NULL;
+		return this->checkable && this->error == INVALID_STRING_ID && this->grfconfig != nullptr;
 	}
 
 	void Clear();
@@ -100,6 +105,7 @@ struct FiosItem {
 	uint64 mtime;
 	char title[64];
 	char name[MAX_PATH];
+	bool operator< (const FiosItem &other) const;
 };
 
 /** List of file information. */
@@ -113,16 +119,17 @@ public:
 	 */
 	inline FiosItem *Append()
 	{
-		return this->files.Append();
+		/*C++17: return &*/ this->files.emplace_back();
+		return &this->files.back();
 	}
 
 	/**
 	 * Get the number of files in the list.
 	 * @return The number of files stored in the list.
 	 */
-	inline uint Length() const
+	inline size_t Length() const
 	{
-		return this->files.Length();
+		return this->files.size();
 	}
 
 	/**
@@ -131,7 +138,7 @@ public:
 	 */
 	inline const FiosItem *Begin() const
 	{
-		return this->files.Begin();
+		return this->files.data();
 	}
 
 	/**
@@ -140,28 +147,28 @@ public:
 	 */
 	inline const FiosItem *End() const
 	{
-		return this->files.End();
+		return this->Begin() + this->Length();
 	}
 
 	/**
 	 * Get a pointer to the indicated file information. File information must exist.
 	 * @return Address of the indicated existing file information.
 	 */
-	inline const FiosItem *Get(uint index) const
+	inline const FiosItem *Get(size_t index) const
 	{
-		return this->files.Get(index);
+		return this->files.data() + index;
 	}
 
 	/**
 	 * Get a pointer to the indicated file information. File information must exist.
 	 * @return Address of the indicated existing file information.
 	 */
-	inline FiosItem *Get(uint index)
+	inline FiosItem *Get(size_t index)
 	{
-		return this->files.Get(index);
+		return this->files.data() + index;
 	}
 
-	inline const FiosItem &operator[](uint index) const
+	inline const FiosItem &operator[](size_t index) const
 	{
 		return this->files[index];
 	}
@@ -170,7 +177,7 @@ public:
 	 * Get a reference to the indicated file information. File information must exist.
 	 * @return The requested file information.
 	 */
-	inline FiosItem &operator[](uint index)
+	inline FiosItem &operator[](size_t index)
 	{
 		return this->files[index];
 	}
@@ -178,19 +185,19 @@ public:
 	/** Remove all items from the list. */
 	inline void Clear()
 	{
-		this->files.Clear();
+		this->files.clear();
 	}
 
 	/** Compact the list down to the smallest block size boundary. */
 	inline void Compact()
 	{
-		this->files.Compact();
+		this->files.shrink_to_fit();
 	}
 
 	void BuildFileList(AbstractFileType abstract_filetype, SaveLoadOperation fop);
 	const FiosItem *FindItem(const char *file);
 
-	SmallVector<FiosItem, 32> files; ///< The list of files.
+	std::vector<FiosItem> files; ///< The list of files.
 };
 
 enum SortingBits {
@@ -219,6 +226,4 @@ void FiosMakeSavegameName(char *buf, const char *name, const char *last);
 
 FiosType FiosGetSavegameListCallback(SaveLoadOperation fop, const char *file, const char *ext, char *title, const char *last);
 
-int CDECL CompareFiosItems(const FiosItem *a, const FiosItem *b);
-
 #endif /* FIOS_H */
