@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -72,7 +70,7 @@ public:
 	{
 		this->Clear();
 		free(this->data);
-		this->data = NULL;
+		this->data = nullptr;
 	}
 
 protected:
