@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -155,8 +153,8 @@ struct DumpTarget {
 	{
 		static size_t type_id = ++LastTypeId();
 
-		if (s == NULL) {
-			/* No need to dump NULL struct. */
+		if (s == nullptr) {
+			/* No need to dump nullptr struct. */
 			WriteLine("%s = <null>", name);
 			return;
 		}
