@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -94,7 +92,7 @@ public:
 		Clear();
 		/* free the memory block occupied by items */
 		free(((byte*)data) - HeaderSize);
-		data = NULL;
+		data = nullptr;
 	}
 
 	/** Clear (destroy) all items */
