@@ -1,5 +1,3 @@
-/* $Id: misc_cmd.cpp 23618 2011-12-19 20:58:59Z truebrain $ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -12,6 +10,7 @@
 #include "stdafx.h"
 #include "command_func.h"
 #include "economy_func.h"
+#include "cmd_helper.h"
 #include "window_func.h"
 #include "textbuf_gui.h"
 #include "network/network.h"
@@ -24,6 +23,8 @@
 
 #include "table/strings.h"
 
+#include "safeguards.h"
+
 /**
  * Increase the loan of your company.
  * @param tile unused
@@ -151,12 +152,10 @@ CommandCost CmdPause(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2,
 		case PM_PAUSED_GAME_SCRIPT:
 			break;
 
-#ifdef ENABLE_NETWORK
 		case PM_PAUSED_JOIN:
 		case PM_PAUSED_ACTIVE_CLIENTS:
 			if (!_networking) return CMD_ERROR;
 			break;
-#endif /* ENABLE_NETWORK */
 
 		default: return CMD_ERROR;
 	}
@@ -165,24 +164,19 @@ CommandCost CmdPause(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2,
 			ShowQuery(
 				STR_NEWGRF_UNPAUSE_WARNING_TITLE,
 				STR_NEWGRF_UNPAUSE_WARNING,
-				NULL,
+				nullptr,
 				AskUnsafeUnpauseCallback
 			);
 		} else {
-#ifdef ENABLE_NETWORK
 			PauseMode prev_mode = _pause_mode;
-#endif /* ENABLE_NETWORK */
 
 			if (p2 == 0) {
-				_pause_mode = _pause_mode & ~p1;
+				_pause_mode = static_cast<PauseMode>(_pause_mode & (byte)~p1);
 			} else {
-				_pause_mode = _pause_mode | p1;
+				_pause_mode = static_cast<PauseMode>(_pause_mode | (byte)p1);
 			}
-			InvalidateWindowClassesData(WC_AI_DEBUG, -2);
 
-#ifdef ENABLE_NETWORK
 			NetworkHandlePauseChange(prev_mode, (PauseMode)p1);
-#endif /* ENABLE_NETWORK */
 		}
 
 		SetWindowDirty(WC_STATUS_BAR, 0);
@@ -195,7 +189,7 @@ CommandCost CmdPause(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2,
  * Change the financial flow of your company.
  * @param tile unused
  * @param flags operation to perform
- * @param p1 the amount of money to receive (if negative), or spend (if positive)
+ * @param p1 the amount of money to receive (if positive), or spend (if negative)
  * @param p2 unused
  * @param text unused
  * @return the cost of this operation or an error
@@ -205,10 +199,42 @@ CommandCost CmdMoneyCheat(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 	return CommandCost(EXPENSES_OTHER, -(int32)p1);
 }
 
+/**
+ * Change the bank bank balance of a company by inserting or removing money without affecting the loan.
+ * @param tile unused
+ * @param flags operation to perform
+ * @param p1 the amount of money to receive (if positive), or spend (if negative)
+ * @param p2 (bit 0-7)  - the company ID.
+ *           (bit 8-15) - the expenses type which should register the cost/income @see ExpensesType.
+ * @param text unused
+ * @return zero cost or an error
+ */
+CommandCost CmdChangeBankBalance(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
+{
+	int32 delta = (int32)p1;
+	CompanyID company = (CompanyID) GB(p2, 0, 8);
+	ExpensesType expenses_type = Extract<ExpensesType, 8, 8>(p2);
+
+	if (!Company::IsValidID(company)) return CMD_ERROR;
+	if (expenses_type >= EXPENSES_END) return CMD_ERROR;
+	if (_current_company != OWNER_DEITY) return CMD_ERROR;
+
+	if (flags & DC_EXEC) {
+		/* Change company bank balance of company. */
+		Backup<CompanyID> cur_company(_current_company, company, FILE_LINE);
+		SubtractMoneyFromCompany(CommandCost(expenses_type, -delta));
+		cur_company.Restore();
+	}
+
+	/* This command doesn't cost anything for deity. */
+	CommandCost zero_cost(expenses_type, 0);
+	return zero_cost;
+}
+
 /**
  * Transfer funds (money) from one company to another.
  * To prevent abuse in multiplayer games you can only send money to other
- * companies if you have paid off your loan (either explicitely, or implicitely
+ * companies if you have paid off your loan (either explicitly, or implicitly
  * given the fact that you have more money than loan).
  * @param tile unused
  * @param flags operation to perform
@@ -231,7 +257,7 @@ CommandCost CmdGiveMoney(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 
 	if (flags & DC_EXEC) {
 		/* Add money to company */
-		Backup<CompanyByte> cur_company(_current_company, dest_company, FILE_LINE);
+		Backup<CompanyID> cur_company(_current_company, dest_company, FILE_LINE);
 		SubtractMoneyFromCompany(CommandCost(EXPENSES_OTHER, -amount.GetCost()));
 		cur_company.Restore();
 	}
