@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -17,25 +15,25 @@
 /** The midi player for BeOS. */
 class MusicDriver_BeMidi : public MusicDriver {
 public:
-	/* virtual */ const char *Start(const char * const *param);
+	const char *Start(const char * const *param) override;
 
-	/* virtual */ void Stop();
+	void Stop() override;
 
-	/* virtual */ void PlaySong(const MusicSongInfo &song);
+	void PlaySong(const MusicSongInfo &song) override;
 
-	/* virtual */ void StopSong();
+	void StopSong() override;
 
-	/* virtual */ bool IsSongPlaying();
+	bool IsSongPlaying() override;
 
-	/* virtual */ void SetVolume(byte vol);
-	/* virtual */ const char *GetName() const { return "bemidi"; }
+	void SetVolume(byte vol) override;
+	const char *GetName() const override { return "bemidi"; }
 };
 
 /** Factory for the BeOS midi player. */
 class FMusicDriver_BeMidi : public DriverFactoryBase {
 public:
 	FMusicDriver_BeMidi() : DriverFactoryBase(Driver::DT_MUSIC, 10, "bemidi", "BeOS MIDI Driver") {}
-	/* virtual */ Driver *CreateInstance() const { return new MusicDriver_BeMidi(); }
+	Driver *CreateInstance() const override { return new MusicDriver_BeMidi(); }
 };
 
 #endif /* MUSIC_BEMIDI_H */
