@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -26,8 +24,6 @@
 #include "../openttd.h"
 #include "../company_type.h"
 
-#ifdef ENABLE_NETWORK
-
 extern NetworkServerGameInfo _network_game_info;
 extern NetworkCompanyState *_network_company_states;
 
@@ -53,7 +49,7 @@ void NetworkPopulateCompanyStats(NetworkCompanyStats *stats);
 
 void NetworkUpdateClientInfo(ClientID client_id);
 void NetworkClientsToSpectators(CompanyID cid);
-void NetworkClientConnectGame(NetworkAddress address, CompanyID join_as, const char *join_server_password = NULL, const char *join_company_password = NULL);
+void NetworkClientConnectGame(NetworkAddress address, CompanyID join_as, const char *join_server_password = nullptr, const char *join_company_password = nullptr);
 void NetworkClientRequestMove(CompanyID company, const char *pass = "");
 void NetworkClientSendRcon(const char *password, const char *command);
 void NetworkClientSendChat(NetworkAction action, DestType type, int dest, const char *msg, int64 data = 0);
@@ -79,9 +75,9 @@ void NetworkServerDoMove(ClientID client_id, CompanyID company_id);
 void NetworkServerSendRcon(ClientID client_id, TextColour colour_code, const char *string);
 void NetworkServerSendChat(NetworkAction action, DestType type, int dest, const char *msg, ClientID from_id, int64 data = 0, bool from_admin = false);
 
-void NetworkServerKickClient(ClientID client_id);
-uint NetworkServerKickOrBanIP(ClientID client_id, bool ban);
-uint NetworkServerKickOrBanIP(const char *ip, bool ban);
+void NetworkServerKickClient(ClientID client_id, const char *reason);
+uint NetworkServerKickOrBanIP(ClientID client_id, bool ban, const char *reason);
+uint NetworkServerKickOrBanIP(const char *ip, bool ban, const char *reason);
 
 void NetworkInitChatMessage();
 void CDECL NetworkAddChatMessage(TextColour colour, uint duration, const char *message, ...) WARN_FORMAT(3, 4);
@@ -90,5 +86,4 @@ void NetworkChatMessageLoop();
 
 void NetworkAfterNewGRFScan();
 
-#endif /* ENABLE_NETWORK */
 #endif /* NETWORK_FUNC_H */
