@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -12,24 +10,20 @@
  * Also, it handles the request to a server for data about the server
  */
 
-#ifdef ENABLE_NETWORK
-
 #include "../stdafx.h"
 #include "../debug.h"
 #include "../window_func.h"
-#include "../thread/thread.h"
 #include "network_internal.h"
 #include "network_udp.h"
 #include "network_gamelist.h"
+#include <atomic>
 
 #include "../safeguards.h"
 
-NetworkGameList *_network_game_list = NULL;
+NetworkGameList *_network_game_list = nullptr;
 
-/** Mutex for handling delayed insertion/querying of servers. */
-static ThreadMutex *_network_game_list_mutex = ThreadMutex::New();
 /** The games to insert when the GUI thread has time for us. */
-static NetworkGameList *_network_game_delayed_insertion_list = NULL;
+static std::atomic<NetworkGameList *> _network_game_delayed_insertion_list(nullptr);
 
 /**
  * Add a new item to the linked gamelist, but do it delayed in the next tick
@@ -38,23 +32,21 @@ static NetworkGameList *_network_game_delayed_insertion_list = NULL;
  */
 void NetworkGameListAddItemDelayed(NetworkGameList *item)
 {
-	_network_game_list_mutex->BeginCritical();
-	item->next = _network_game_delayed_insertion_list;
-	_network_game_delayed_insertion_list = item;
-	_network_game_list_mutex->EndCritical();
+	item->next = _network_game_delayed_insertion_list.load(std::memory_order_relaxed);
+	while (!_network_game_delayed_insertion_list.compare_exchange_weak(item->next, item, std::memory_order_acq_rel)) {}
 }
 
 /** Perform the delayed (thread safe) insertion into the game list */
 static void NetworkGameListHandleDelayedInsert()
 {
-	_network_game_list_mutex->BeginCritical();
-	while (_network_game_delayed_insertion_list != NULL) {
-		NetworkGameList *ins_item = _network_game_delayed_insertion_list;
-		_network_game_delayed_insertion_list = ins_item->next;
+	while (true) {
+		NetworkGameList *ins_item = _network_game_delayed_insertion_list.load(std::memory_order_relaxed);
+		while (ins_item != nullptr && !_network_game_delayed_insertion_list.compare_exchange_weak(ins_item, ins_item->next, std::memory_order_acq_rel)) {}
+		if (ins_item == nullptr) break; // No item left.
 
 		NetworkGameList *item = NetworkGameListAddItem(ins_item->address);
 
-		if (item != NULL) {
+		if (item != nullptr) {
 			if (StrEmpty(item->info.server_name)) {
 				ClearGRFConfigList(&item->info.grfconfig);
 				memset(&item->info, 0, sizeof(item->info));
@@ -68,7 +60,6 @@ static void NetworkGameListHandleDelayedInsert()
 		}
 		free(ins_item);
 	}
-	_network_game_list_mutex->EndCritical();
 }
 
 /**
@@ -85,22 +76,22 @@ NetworkGameList *NetworkGameListAddItem(NetworkAddress address)
 	if (StrEmpty(hostname) ||
 			strcmp(hostname, "0.0.0.0") == 0 ||
 			strcmp(hostname, "::") == 0) {
-		return NULL;
+		return nullptr;
 	}
 
 	NetworkGameList *item, *prev_item;
 
-	prev_item = NULL;
-	for (item = _network_game_list; item != NULL; item = item->next) {
+	prev_item = nullptr;
+	for (item = _network_game_list; item != nullptr; item = item->next) {
 		if (item->address == address) return item;
 		prev_item = item;
 	}
 
 	item = CallocT<NetworkGameList>(1);
-	item->next = NULL;
+	item->next = nullptr;
 	item->address = address;
 
-	if (prev_item == NULL) {
+	if (prev_item == nullptr) {
 		_network_game_list = item;
 	} else {
 		prev_item->next = item;
@@ -118,10 +109,10 @@ NetworkGameList *NetworkGameListAddItem(NetworkAddress address)
  */
 void NetworkGameListRemoveItem(NetworkGameList *remove)
 {
-	NetworkGameList *prev_item = NULL;
-	for (NetworkGameList *item = _network_game_list; item != NULL; item = item->next) {
+	NetworkGameList *prev_item = nullptr;
+	for (NetworkGameList *item = _network_game_list; item != nullptr; item = item->next) {
 		if (remove == item) {
-			if (prev_item == NULL) {
+			if (prev_item == nullptr) {
 				_network_game_list = remove->next;
 			} else {
 				prev_item->next = remove->next;
@@ -130,7 +121,7 @@ void NetworkGameListRemoveItem(NetworkGameList *remove)
 			/* Remove GRFConfig information */
 			ClearGRFConfigList(&remove->info.grfconfig);
 			free(remove);
-			remove = NULL;
+			remove = nullptr;
 
 			DEBUG(net, 4, "[gamelist] removed server from list");
 			NetworkRebuildHostList();
@@ -155,7 +146,7 @@ void NetworkGameListRequery()
 	if (++requery_cnt < REQUERY_EVERY_X_GAMELOOPS) return;
 	requery_cnt = 0;
 
-	for (NetworkGameList *item = _network_game_list; item != NULL; item = item->next) {
+	for (NetworkGameList *item = _network_game_list; item != nullptr; item = item->next) {
 		item->retries++;
 		if (item->retries < REFRESH_GAMEINFO_X_REQUERIES && (item->online || item->retries >= MAX_GAME_LIST_REQUERY_COUNT)) continue;
 
@@ -172,15 +163,15 @@ void NetworkGameListRequery()
  */
 void NetworkAfterNewGRFScan()
 {
-	for (NetworkGameList *item = _network_game_list; item != NULL; item = item->next) {
+	for (NetworkGameList *item = _network_game_list; item != nullptr; item = item->next) {
 		/* Reset compatibility state */
 		item->info.compatible = item->info.version_compatible;
 
-		for (GRFConfig *c = item->info.grfconfig; c != NULL; c = c->next) {
+		for (GRFConfig *c = item->info.grfconfig; c != nullptr; c = c->next) {
 			assert(HasBit(c->flags, GCF_COPY));
 
 			const GRFConfig *f = FindGRFConfig(c->ident.grfid, FGCM_EXACT, c->ident.md5sum);
-			if (f == NULL) {
+			if (f == nullptr) {
 				/* Don't know the GRF, so mark game incompatible and the (possibly)
 				 * already resolved name for this GRF (another server has sent the
 				 * name of the GRF already. */
@@ -206,5 +197,3 @@ void NetworkAfterNewGRFScan()
 
 	InvalidateWindowClassesData(WC_NETWORK_WINDOW);
 }
-
-#endif /* ENABLE_NETWORK */
