@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -12,8 +10,6 @@
 #ifndef NETWORK_UDP_H
 #define NETWORK_UDP_H
 
-#ifdef ENABLE_NETWORK
-
 #include "core/address.h"
 
 void NetworkUDPInitialize();
@@ -25,6 +21,4 @@ void NetworkUDPRemoveAdvertise(bool blocking);
 void NetworkUDPClose();
 void NetworkBackgroundUDPLoop();
 
-#endif /* ENABLE_NETWORK */
-
 #endif /* NETWORK_UDP_H */
