@@ -1,5 +1,3 @@
-/* $Id: newgrf_commons.cpp 23740 2012-01-03 21:32:51Z rubidium $ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -32,6 +30,8 @@
 
 #include "table/strings.h"
 
+#include "safeguards.h"
+
 /**
  * Constructor of generic class
  * @param offset end of original data for this entity. i.e: houses = 110
@@ -95,7 +95,7 @@ void OverrideManagerBase::ResetOverride()
 
 /**
  * Return the ID (if ever available) of a previously inserted entity.
- * @param grf_local_id ID of this enity withing the grfID
+ * @param grf_local_id ID of this entity within the grfID
  * @param grfid ID of the grf file
  * @return the ID of the candidate, of the Invalid flag item ID
  */
@@ -170,7 +170,7 @@ uint16 OverrideManagerBase::GetSubstituteID(uint16 entity_id) const
 
 /**
  * Install the specs into the HouseSpecs array
- * It will find itself the proper slot onwhich it will go
+ * It will find itself the proper slot on which it will go
  * @param hs HouseSpec read from the grf file, ready for inclusion
  */
 void HouseOverrideManager::SetEntitySpec(const HouseSpec *hs)
@@ -198,7 +198,7 @@ void HouseOverrideManager::SetEntitySpec(const HouseSpec *hs)
 
 /**
  * Return the ID (if ever available) of a previously inserted entity.
- * @param grf_local_id ID of this enity withing the grfID
+ * @param grf_local_id ID of this entity within the grfID
  * @param grfid ID of the grf file
  * @return the ID of the candidate, of the Invalid flag item ID
  */
@@ -226,16 +226,16 @@ uint16 IndustryOverrideManager::AddEntityID(byte grf_local_id, uint32 grfid, byt
 {
 	/* This entity hasn't been defined before, so give it an ID now. */
 	for (uint16 id = 0; id < max_new_entities; id++) {
-		/* Skip overriden industries */
+		/* Skip overridden industries */
 		if (id < max_offset && entity_overrides[id] != invalid_ID) continue;
 
 		/* Get the real live industry */
 		const IndustrySpec *inds = GetIndustrySpec(id);
 
 		/* This industry must be one that is not available(enabled), mostly because of climate.
-		 * And it must not already be used by a grf (grffile == NULL).
-		 * So reseve this slot here, as it is the chosen one */
-		if (!inds->enabled && inds->grf_prop.grffile == NULL) {
+		 * And it must not already be used by a grf (grffile == nullptr).
+		 * So reserve this slot here, as it is the chosen one */
+		if (!inds->enabled && inds->grf_prop.grffile == nullptr) {
 			EntityIDMapping *map = &mapping_ID[id];
 
 			if (map->entity_id == 0 && map->grfid == 0) {
@@ -252,8 +252,8 @@ uint16 IndustryOverrideManager::AddEntityID(byte grf_local_id, uint32 grfid, byt
 }
 
 /**
- * Method to install the new indistry data in its proper slot
- * The slot assigment is internal of this method, since it requires
+ * Method to install the new industry data in its proper slot
+ * The slot assignment is internal of this method, since it requires
  * checking what is available
  * @param inds Industryspec that comes from the grf decoding process
  */
@@ -264,11 +264,11 @@ void IndustryOverrideManager::SetEntitySpec(IndustrySpec *inds)
 
 	if (ind_id == invalid_ID) {
 		/* Not found.
-		 * Or it has already been overriden, so you've lost your place old boy.
+		 * Or it has already been overridden, so you've lost your place old boy.
 		 * Or it is a simple substitute.
 		 * We need to find a free available slot */
 		ind_id = this->AddEntityID(inds->grf_prop.local_id, inds->grf_prop.grffile->grfid, inds->grf_prop.subst_id);
-		inds->grf_prop.override = invalid_ID;  // make sure it will not be detected as overriden
+		inds->grf_prop.override = invalid_ID;  // make sure it will not be detected as overridden
 	}
 
 	if (ind_id == invalid_ID) {
@@ -277,7 +277,7 @@ void IndustryOverrideManager::SetEntitySpec(IndustrySpec *inds)
 	}
 
 	/* Now that we know we can use the given id, copy the spec to its final destination... */
-	memcpy(&_industry_specs[ind_id], inds, sizeof(*inds));
+	_industry_specs[ind_id] = *inds;
 	/* ... and mark it as usable*/
 	_industry_specs[ind_id].enabled = true;
 }
@@ -308,7 +308,7 @@ void IndustryTileOverrideManager::SetEntitySpec(const IndustryTileSpec *its)
 
 /**
  * Method to install the new object data in its proper slot
- * The slot assigment is internal of this method, since it requires
+ * The slot assignment is internal of this method, since it requires
  * checking what is available
  * @param spec ObjectSpec that comes from the grf decoding process
  */
@@ -319,7 +319,7 @@ void ObjectOverrideManager::SetEntitySpec(ObjectSpec *spec)
 
 	if (type == invalid_ID) {
 		/* Not found.
-		 * Or it has already been overriden, so you've lost your place old boy.
+		 * Or it has already been overridden, so you've lost your place old boy.
 		 * Or it is a simple substitute.
 		 * We need to find a free available slot */
 		type = this->AddEntityID(spec->grf_prop.local_id, spec->grf_prop.grffile->grfid, OBJECT_TRANSMITTER);
@@ -461,28 +461,28 @@ uint32 GetNearbyTileInformation(TileIndex tile, bool grf_version8)
 /**
  * Returns company information like in vehicle var 43 or station var 43.
  * @param owner Owner of the object.
- * @param l Livery of the object; NULL to use default.
+ * @param l Livery of the object; nullptr to use default.
  * @return NewGRF company information.
  */
 uint32 GetCompanyInfo(CompanyID owner, const Livery *l)
 {
-	if (l == NULL && Company::IsValidID(owner)) l = &Company::Get(owner)->livery[LS_DEFAULT];
-	return owner | (Company::IsValidAiID(owner) ? 0x10000 : 0) | (l != NULL ? (l->colour1 << 24) | (l->colour2 << 28) : 0);
+	if (l == nullptr && Company::IsValidID(owner)) l = &Company::Get(owner)->livery[LS_DEFAULT];
+	return owner | (Company::IsValidAiID(owner) ? 0x10000 : 0) | (l != nullptr ? (l->colour1 << 24) | (l->colour2 << 28) : 0);
 }
 
 /**
  * Get the error message from a shape/location/slope check callback result.
  * @param cb_res Callback result to translate. If bit 10 is set this is a standard error message, otherwise a NewGRF provided string.
- * @param grfid grfID to use to resolve a custom error message.
+ * @param grffile NewGRF to use to resolve a custom error message.
  * @param default_error Error message to use for the generic error.
  * @return CommandCost indicating success or the error message.
  */
-CommandCost GetErrorMessageFromLocationCallbackResult(uint16 cb_res, uint32 grfid, StringID default_error)
+CommandCost GetErrorMessageFromLocationCallbackResult(uint16 cb_res, const GRFFile *grffile, StringID default_error)
 {
 	CommandCost res;
 
 	if (cb_res < 0x400) {
-		res = CommandCost(GetGRFStringID(grfid, 0xD000 + cb_res));
+		res = CommandCost(GetGRFStringID(grffile->grfid, 0xD000 + cb_res));
 	} else {
 		switch (cb_res) {
 			case 0x400: return res; // No error.
@@ -501,7 +501,7 @@ CommandCost GetErrorMessageFromLocationCallbackResult(uint16 cb_res, uint32 grfi
 	}
 
 	/* Copy some parameters from the registers to the error message text ref. stack */
-	res.UseTextRefStack(4);
+	res.UseTextRefStack(grffile, 4);
 
 	return res;
 }
@@ -577,7 +577,7 @@ bool Convert8bitBooleanCallback(const GRFFile *grffile, uint16 cbid, uint16 cb_r
 }
 
 
-/* static */ SmallVector<DrawTileSeqStruct, 8> NewGRFSpriteLayout::result_seq;
+/* static */ std::vector<DrawTileSeqStruct> NewGRFSpriteLayout::result_seq;
 
 /**
  * Clone the building sprites of a spritelayout.
@@ -585,8 +585,8 @@ bool Convert8bitBooleanCallback(const GRFFile *grffile, uint16 cbid, uint16 cb_r
  */
 void NewGRFSpriteLayout::Clone(const DrawTileSeqStruct *source)
 {
-	assert(this->seq == NULL);
-	assert(source != NULL);
+	assert(this->seq == nullptr);
+	assert(source != nullptr);
 
 	size_t count = 1; // 1 for the terminator
 	const DrawTileSeqStruct *element;
@@ -605,7 +605,7 @@ void NewGRFSpriteLayout::Clone(const NewGRFSpriteLayout *source)
 {
 	this->Clone((const DrawTileSprites*)source);
 
-	if (source->registers != NULL) {
+	if (source->registers != nullptr) {
 		size_t count = 1; // 1 for the ground sprite
 		const DrawTileSeqStruct *element;
 		foreach_draw_tile_seq(element, source->seq) count++;
@@ -623,7 +623,7 @@ void NewGRFSpriteLayout::Clone(const NewGRFSpriteLayout *source)
  */
 void NewGRFSpriteLayout::Allocate(uint num_sprites)
 {
-	assert(this->seq == NULL);
+	assert(this->seq == nullptr);
 
 	DrawTileSeqStruct *sprites = CallocT<DrawTileSeqStruct>(num_sprites + 1);
 	sprites[num_sprites].MakeTerminator();
@@ -635,8 +635,8 @@ void NewGRFSpriteLayout::Allocate(uint num_sprites)
  */
 void NewGRFSpriteLayout::AllocateRegisters()
 {
-	assert(this->seq != NULL);
-	assert(this->registers == NULL);
+	assert(this->seq != nullptr);
+	assert(this->registers == nullptr);
 
 	size_t count = 1; // 1 for the ground sprite
 	const DrawTileSeqStruct *element;
@@ -648,7 +648,7 @@ void NewGRFSpriteLayout::AllocateRegisters()
 /**
  * Prepares a sprite layout before resolving action-1-2-3 chains.
  * Integrates offsets into the layout and determines which chains to resolve.
- * @note The function uses statically allocated temporary storage, which is reused everytime when calling the function.
+ * @note The function uses statically allocated temporary storage, which is reused every time when calling the function.
  *       That means, you have to use the sprite layout before calling #PrepareLayout() the next time.
  * @param orig_offset          Offset to apply to non-action-1 sprites.
  * @param newgrf_ground_offset Offset to apply to action-1 ground sprites.
@@ -659,12 +659,13 @@ void NewGRFSpriteLayout::AllocateRegisters()
  */
 uint32 NewGRFSpriteLayout::PrepareLayout(uint32 orig_offset, uint32 newgrf_ground_offset, uint32 newgrf_offset, uint constr_stage, bool separate_ground) const
 {
-	result_seq.Clear();
+	result_seq.clear();
 	uint32 var10_values = 0;
 
 	/* Create a copy of the spritelayout, so we can modify some values.
 	 * Also include the groundsprite into the sequence for easier processing. */
-	DrawTileSeqStruct *result = result_seq.Append();
+	/*C++17: DrawTileSeqStruct *result = &*/ result_seq.emplace_back();
+	DrawTileSeqStruct *result = &result_seq.back();
 	result->image = ground;
 	result->delta_x = 0;
 	result->delta_y = 0;
@@ -672,17 +673,17 @@ uint32 NewGRFSpriteLayout::PrepareLayout(uint32 orig_offset, uint32 newgrf_groun
 
 	const DrawTileSeqStruct *dtss;
 	foreach_draw_tile_seq(dtss, this->seq) {
-		*result_seq.Append() = *dtss;
+		result_seq.push_back(*dtss);
 	}
-	result_seq.Append()->MakeTerminator();
-
+	result_seq.emplace_back() /*C++17: .MakeTerminator()*/;
+	result_seq.back().MakeTerminator();
 	/* Determine the var10 values the action-1-2-3 chains needs to be resolved for,
 	 * and apply the default sprite offsets (unless disabled). */
 	const TileLayoutRegisters *regs = this->registers;
 	bool ground = true;
-	foreach_draw_tile_seq(result, result_seq.Begin()) {
+	foreach_draw_tile_seq(result, result_seq.data()) {
 		TileLayoutFlags flags = TLF_NOTHING;
-		if (regs != NULL) flags = regs->flags;
+		if (regs != nullptr) flags = regs->flags;
 
 		/* Record var10 value for the sprite */
 		if (HasBit(result->image.sprite, SPRITE_MODIFIER_CUSTOM_SPRITE) || (flags & TLF_SPRITE_REG_FLAGS)) {
@@ -694,7 +695,7 @@ uint32 NewGRFSpriteLayout::PrepareLayout(uint32 orig_offset, uint32 newgrf_groun
 		if (!(flags & TLF_SPRITE)) {
 			if (HasBit(result->image.sprite, SPRITE_MODIFIER_CUSTOM_SPRITE)) {
 				result->image.sprite += ground ? newgrf_ground_offset : newgrf_offset;
-				if (constr_stage > 0 && regs != NULL) result->image.sprite += GetConstructionStageOffset(constr_stage, regs->max_sprite_offset);
+				if (constr_stage > 0 && regs != nullptr) result->image.sprite += GetConstructionStageOffset(constr_stage, regs->max_sprite_offset);
 			} else {
 				result->image.sprite += orig_offset;
 			}
@@ -710,12 +711,12 @@ uint32 NewGRFSpriteLayout::PrepareLayout(uint32 orig_offset, uint32 newgrf_groun
 		if (!(flags & TLF_PALETTE)) {
 			if (HasBit(result->image.pal, SPRITE_MODIFIER_CUSTOM_SPRITE)) {
 				result->image.sprite += ground ? newgrf_ground_offset : newgrf_offset;
-				if (constr_stage > 0 && regs != NULL) result->image.sprite += GetConstructionStageOffset(constr_stage, regs->max_palette_offset);
+				if (constr_stage > 0 && regs != nullptr) result->image.sprite += GetConstructionStageOffset(constr_stage, regs->max_palette_offset);
 			}
 		}
 
 		ground = false;
-		if (regs != NULL) regs++;
+		if (regs != nullptr) regs++;
 	}
 
 	return var10_values;
@@ -734,9 +735,9 @@ void NewGRFSpriteLayout::ProcessRegisters(uint8 resolved_var10, uint32 resolved_
 	DrawTileSeqStruct *result;
 	const TileLayoutRegisters *regs = this->registers;
 	bool ground = true;
-	foreach_draw_tile_seq(result, result_seq.Begin()) {
+	foreach_draw_tile_seq(result, result_seq.data()) {
 		TileLayoutFlags flags = TLF_NOTHING;
-		if (regs != NULL) flags = regs->flags;
+		if (regs != nullptr) flags = regs->flags;
 
 		/* Is the sprite or bounding box affected by an action-1-2-3 chain? */
 		if (HasBit(result->image.sprite, SPRITE_MODIFIER_CUSTOM_SPRITE) || (flags & TLF_SPRITE_REG_FLAGS)) {
@@ -791,6 +792,6 @@ void NewGRFSpriteLayout::ProcessRegisters(uint8 resolved_var10, uint32 resolved_
 		}
 
 		ground = false;
-		if (regs != NULL) regs++;
+		if (regs != nullptr) regs++;
 	}
 }
