@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -23,6 +21,7 @@
 #include "station_base.h"
 #include "company_base.h"
 #include "newgrf_railtype.h"
+#include "newgrf_roadtype.h"
 #include "ship.h"
 
 #include "safeguards.h"
@@ -65,7 +64,7 @@ const SpriteGroup *GetWagonOverrideSpriteSet(EngineID engine, CargoID cargo, Eng
 			if (wo->train_id[j] == overriding_engine) return wo->group;
 		}
 	}
-	return NULL;
+	return nullptr;
 }
 
 /**
@@ -79,7 +78,7 @@ void UnloadWagonOverrides(Engine *e)
 	}
 	free(e->overrides);
 	e->overrides_count = 0;
-	e->overrides = NULL;
+	e->overrides = nullptr;
 }
 
 
@@ -88,7 +87,7 @@ void SetCustomEngineSprites(EngineID engine, byte cargo, const SpriteGroup *grou
 	Engine *e = Engine::Get(engine);
 	assert(cargo < lengthof(e->grf_prop.spritegroup));
 
-	if (e->grf_prop.spritegroup[cargo] != NULL) {
+	if (e->grf_prop.spritegroup[cargo] != nullptr) {
 		grfmsg(6, "SetCustomEngineSprites: engine %d cargo %d already has group -- replacing", engine, cargo);
 	}
 	e->grf_prop.spritegroup[cargo] = group;
@@ -166,7 +165,7 @@ enum TTDPAircraftMovementStates {
 static byte MapAircraftMovementState(const Aircraft *v)
 {
 	const Station *st = GetTargetAirportIfValid(v);
-	if (st == NULL) return AMS_TTDP_FLIGHT_TO_TOWER;
+	if (st == nullptr) return AMS_TTDP_FLIGHT_TO_TOWER;
 
 	const AirportFTAClass *afc = st->airport.GetFTA();
 	uint16 amdflag = afc->MovingData(v->pos)->flag;
@@ -247,7 +246,7 @@ static byte MapAircraftMovementState(const Aircraft *v)
 			return AMS_TTDP_TO_INWAY;
 
 		case HELILANDING:
-		case HELIENDLANDING: // Helicoptor is decending.
+		case HELIENDLANDING: // Helicoptor is descending.
 			if (amdflag & AMED_HELI_LOWER) {
 				return afc->delta_z == 0 ?
 					AMS_TTDP_HELI_LAND_AIRPORT : AMS_TTDP_HELI_LAND_HELIPORT;
@@ -339,12 +338,12 @@ static byte MapAircraftMovementAction(const Aircraft *v)
 
 /* virtual */ uint32 VehicleScopeResolver::GetRandomBits() const
 {
-	return this->v == NULL ? 0 : this->v->random_bits;
+	return this->v == nullptr ? 0 : this->v->random_bits;
 }
 
 /* virtual */ uint32 VehicleScopeResolver::GetTriggers() const
 {
-	return this->v == NULL ? 0 : this->v->waiting_triggers;
+	return this->v == nullptr ? 0 : this->v->waiting_triggers;
 }
 
 
@@ -355,12 +354,12 @@ static byte MapAircraftMovementAction(const Aircraft *v)
 		case VSG_SCOPE_PARENT: return &this->parent_scope;
 		case VSG_SCOPE_RELATIVE: {
 			int32 count = GB(relative, 0, 4);
-			if (this->self_scope.v != NULL && (relative != this->cached_relative_count || count == 0)) {
+			if (this->self_scope.v != nullptr && (relative != this->cached_relative_count || count == 0)) {
 				/* Note: This caching only works as long as the VSG_SCOPE_RELATIVE cannot be used in
 				 *       VarAct2 with procedure calls. */
 				if (count == 0) count = GetRegister(0x100);
 
-				const Vehicle *v = NULL;
+				const Vehicle *v = nullptr;
 				switch (GB(relative, 6, 2)) {
 					default: NOT_REACHED();
 					case 0x00: // count back (away from the engine), starting at this vehicle
@@ -377,12 +376,12 @@ static byte MapAircraftMovementAction(const Aircraft *v)
 						const Vehicle *self = this->self_scope.v;
 						for (const Vehicle *u = self->First(); u != self; u = u->Next()) {
 							if (u->engine_type != self->engine_type) {
-								v = NULL;
+								v = nullptr;
 							} else {
-								if (v == NULL) v = u;
+								if (v == nullptr) v = u;
 							}
 						}
-						if (v == NULL) v = self;
+						if (v == nullptr) v = self;
 						break;
 					}
 				}
@@ -400,16 +399,16 @@ static byte MapAircraftMovementAction(const Aircraft *v)
  * This always uses dual company colours independent of GUI settings. So it is desync-safe.
  *
  * @param engine Engine type
- * @param v Vehicle, NULL in purchase list.
+ * @param v Vehicle, nullptr in purchase list.
  * @return Livery to use
  */
 static const Livery *LiveryHelper(EngineID engine, const Vehicle *v)
 {
 	const Livery *l;
 
-	if (v == NULL) {
-		if (!Company::IsValidID(_current_company)) return NULL;
-		l = GetEngineLivery(engine, _current_company, INVALID_ENGINE, NULL, LIT_ALL);
+	if (v == nullptr) {
+		if (!Company::IsValidID(_current_company)) return nullptr;
+		l = GetEngineLivery(engine, _current_company, INVALID_ENGINE, nullptr, LIT_ALL);
 	} else if (v->IsGroundVehicle()) {
 		l = GetEngineLivery(v->engine_type, v->owner, v->GetGroundVehicleCache()->first_engine, v, LIT_ALL);
 	} else {
@@ -437,7 +436,7 @@ static uint32 PositionHelper(const Vehicle *v, bool consecutive)
 		if (consecutive && u->engine_type != v->engine_type) chain_before = 0;
 	}
 
-	while (u->Next() != NULL && (!consecutive || u->Next()->engine_type == v->engine_type)) {
+	while (u->Next() != nullptr && (!consecutive || u->Next()->engine_type == v->engine_type)) {
 		chain_after++;
 		u = u->Next();
 	}
@@ -480,7 +479,7 @@ static uint32 VehicleGetVariable(Vehicle *v, const VehicleScopeResolver *object,
 				memset(common_cargoes, 0, sizeof(common_cargoes));
 				memset(common_subtypes, 0, sizeof(common_subtypes));
 
-				for (u = v; u != NULL; u = u->Next()) {
+				for (u = v; u != nullptr; u = u->Next()) {
 					if (v->type == VEH_TRAIN) user_def_data |= Train::From(u)->tcache.user_def_data;
 
 					/* Skip empty engines */
@@ -500,7 +499,7 @@ static uint32 VehicleGetVariable(Vehicle *v, const VehicleScopeResolver *object,
 				}
 
 				/* Count subcargo types of common_cargo_type */
-				for (u = v; u != NULL; u = u->Next()) {
+				for (u = v; u != nullptr; u = u->Next()) {
 					/* Skip empty engines and engines not carrying common_cargo_type */
 					if (u->cargo_type != common_cargo_type || !u->GetEngine()->CanCarryCargo()) continue;
 
@@ -530,12 +529,12 @@ static uint32 VehicleGetVariable(Vehicle *v, const VehicleScopeResolver *object,
 			 *  - For translating the cargo type we need to use the GRF which is resolving the variable, which
 			 *    is object->ro.grffile.
 			 *    In case of CBID_TRAIN_ALLOW_WAGON_ATTACH this is not the same as v->GetGRF().
-			 *  - The grffile == NULL case only happens if this function is called for default vehicles.
+			 *  - The grffile == nullptr case only happens if this function is called for default vehicles.
 			 *    And this is only done by CheckCaches().
 			 */
 			const GRFFile *grffile = object->ro.grffile;
 			uint8 common_bitnum = (common_cargo_type == CT_INVALID) ? 0xFF :
-				(grffile == NULL || grffile->grf_version < 8) ? CargoSpec::Get(common_cargo_type)->bitnum : grffile->cargo_map[common_cargo_type];
+				(grffile == nullptr || grffile->grf_version < 8) ? CargoSpec::Get(common_cargo_type)->bitnum : grffile->cargo_map[common_cargo_type];
 
 			return (v->grf_cache.consist_cargo_information & 0xFFFF00FF) | common_bitnum << 8;
 		}
@@ -557,7 +556,7 @@ static uint32 VehicleGetVariable(Vehicle *v, const VehicleScopeResolver *object,
 
 				const Station *st = GetTargetAirportIfValid(Aircraft::From(v));
 
-				if (st != NULL && st->airport.tile != INVALID_TILE) {
+				if (st != nullptr && st->airport.tile != INVALID_TILE) {
 					airporttype = st->airport.GetSpec()->ttd_airport_type;
 				}
 
@@ -574,8 +573,8 @@ static uint32 VehicleGetVariable(Vehicle *v, const VehicleScopeResolver *object,
 
 			const Vehicle *u_p = v->Previous();
 			const Vehicle *u_n = v->Next();
-			DirDiff f = (u_p == NULL) ?  DIRDIFF_SAME : DirDifference(u_p->direction, v->direction);
-			DirDiff b = (u_n == NULL) ?  DIRDIFF_SAME : DirDifference(v->direction, u_n->direction);
+			DirDiff f = (u_p == nullptr) ?  DIRDIFF_SAME : DirDifference(u_p->direction, v->direction);
+			DirDiff b = (u_n == nullptr) ?  DIRDIFF_SAME : DirDifference(v->direction, u_n->direction);
 			DirDiff t = ChangeDirDiff(f, b);
 
 			return ((t > DIRDIFF_REVERSE ? t | 8 : t) << 16) |
@@ -606,11 +605,21 @@ static uint32 VehicleGetVariable(Vehicle *v, const VehicleScopeResolver *object,
 		case 0x48: return v->GetEngine()->flags; // Vehicle Type Info
 		case 0x49: return v->build_year;
 
-		case 0x4A: {
-			if (v->type != VEH_TRAIN) return 0;
-			RailType rt = GetTileRailType(v->tile);
-			return (HasPowerOnRail(Train::From(v)->railtype, rt) ? 0x100 : 0) | GetReverseRailTypeTranslation(rt, object->ro.grffile);
-		}
+		case 0x4A:
+			switch (v->type) {
+				case VEH_TRAIN: {
+					RailType rt = GetTileRailType(v->tile);
+					return (HasPowerOnRail(Train::From(v)->railtype, rt) ? 0x100 : 0) | GetReverseRailTypeTranslation(rt, object->ro.grffile);
+				}
+
+				case VEH_ROAD: {
+					RoadType rt = GetRoadType(v->tile, GetRoadTramType(RoadVehicle::From(v)->roadtype));
+					return 0x100 | GetReverseRoadTypeTranslation(rt, object->ro.grffile);
+				}
+
+				default:
+					return 0;
+			}
 
 		case 0x4B: // Long date of last service
 			return v->date_of_last_service;
@@ -636,7 +645,7 @@ static uint32 VehicleGetVariable(Vehicle *v, const VehicleScopeResolver *object,
 
 			{
 				uint count = 0;
-				for (; v != NULL; v = v->Next()) {
+				for (; v != nullptr; v = v->Next()) {
 					if (v->GetEngine()->grf_prop.local_id == parameter) count++;
 				}
 				return count;
@@ -653,7 +662,7 @@ static uint32 VehicleGetVariable(Vehicle *v, const VehicleScopeResolver *object,
 					object->ro.callback == CBID_VEHICLE_START_STOP_CHECK || object->ro.callback == CBID_VEHICLE_32DAY_CALLBACK || object->ro.callback == CBID_VEHICLE_COLOUR_MAPPING ||
 					object->ro.callback == CBID_VEHICLE_SPAWN_VISUAL_EFFECT) {
 				Vehicle *u = v->Move((int32)GetRegister(0x10F));
-				if (u == NULL) return 0; // available, but zero
+				if (u == nullptr) return 0; // available, but zero
 
 				if (parameter == 0x5F) {
 					/* This seems to be the only variable that makes sense to access via var 61, but is not handled by VehicleGetVariable */
@@ -676,7 +685,7 @@ static uint32 VehicleGetVariable(Vehicle *v, const VehicleScopeResolver *object,
 			if (!v->IsGroundVehicle()) return 0;
 
 			const Vehicle *u = v->Move((int8)parameter);
-			if (u == NULL) return 0;
+			if (u == nullptr) return 0;
 
 			/* Get direction difference. */
 			bool prev = (int8)parameter < 0;
@@ -810,7 +819,7 @@ static uint32 VehicleGetVariable(Vehicle *v, const VehicleScopeResolver *object,
 		case 0x57: return GB(ClampToI32(v->GetDisplayProfitLastYear()),  8, 24);
 		case 0x58: return GB(ClampToI32(v->GetDisplayProfitLastYear()), 16, 16);
 		case 0x59: return GB(ClampToI32(v->GetDisplayProfitLastYear()), 24,  8);
-		case 0x5A: return v->Next() == NULL ? INVALID_VEHICLE : v->Next()->index;
+		case 0x5A: return v->Next() == nullptr ? INVALID_VEHICLE : v->Next()->index;
 		case 0x5C: return ClampToI32(v->value);
 		case 0x5D: return GB(ClampToI32(v->value),  8, 24);
 		case 0x5E: return GB(ClampToI32(v->value), 16, 16);
@@ -882,10 +891,10 @@ static uint32 VehicleGetVariable(Vehicle *v, const VehicleScopeResolver *object,
 
 /* virtual */ uint32 VehicleScopeResolver::GetVariable(byte variable, uint32 parameter, bool *available) const
 {
-	if (this->v == NULL) {
+	if (this->v == nullptr) {
 		/* Vehicle does not exist, so we're in a purchase list */
 		switch (variable) {
-			case 0x43: return GetCompanyInfo(_current_company, LiveryHelper(this->self_type, NULL)); // Owner information
+			case 0x43: return GetCompanyInfo(_current_company, LiveryHelper(this->self_type, nullptr)); // Owner information
 			case 0x46: return 0;               // Motion counter
 			case 0x47: { // Vehicle cargo info
 				const Engine *e = Engine::Get(this->self_type);
@@ -919,17 +928,17 @@ static uint32 VehicleGetVariable(Vehicle *v, const VehicleScopeResolver *object,
 {
 	const Vehicle *v = this->self_scope.v;
 
-	if (v == NULL) {
+	if (v == nullptr) {
 		if (group->num_loading > 0) return group->loading[0];
 		if (group->num_loaded  > 0) return group->loaded[0];
-		return NULL;
+		return nullptr;
 	}
 
 	bool in_motion = !v->First()->current_order.IsType(OT_LOADING);
 
 	uint totalsets = in_motion ? group->num_loaded : group->num_loading;
 
-	if (totalsets == 0) return NULL;
+	if (totalsets == 0) return nullptr;
 
 	uint set = (v->cargo.StoredCount() * totalsets) / max((uint16)1, v->cargo_cap);
 	set = min(set, totalsets - 1);
@@ -937,6 +946,22 @@ static uint32 VehicleGetVariable(Vehicle *v, const VehicleScopeResolver *object,
 	return in_motion ? group->loaded[set] : group->loading[set];
 }
 
+GrfSpecFeature VehicleResolverObject::GetFeature() const
+{
+	switch (Engine::Get(this->self_scope.self_type)->type) {
+		case VEH_TRAIN: return GSF_TRAINS;
+		case VEH_ROAD: return GSF_ROADVEHICLES;
+		case VEH_SHIP: return GSF_SHIPS;
+		case VEH_AIRCRAFT: return GSF_AIRCRAFT;
+		default: return GSF_INVALID;
+	}
+}
+
+uint32 VehicleResolverObject::GetDebugID() const
+{
+	return Engine::Get(this->self_scope.self_type)->grf_prop.local_id;
+}
+
 /**
  * Get the grf file associated with an engine type.
  * @param engine_type Engine to query.
@@ -945,7 +970,7 @@ static uint32 VehicleGetVariable(Vehicle *v, const VehicleScopeResolver *object,
 static const GRFFile *GetEngineGrfFile(EngineID engine_type)
 {
 	const Engine *e = Engine::Get(engine_type);
-	return (e != NULL) ? e->GetGRF() : NULL;
+	return (e != nullptr) ? e->GetGRF() : nullptr;
 }
 
 /**
@@ -962,14 +987,14 @@ VehicleResolverObject::VehicleResolverObject(EngineID engine_type, const Vehicle
 		CallbackID callback, uint32 callback_param1, uint32 callback_param2)
 	: ResolverObject(GetEngineGrfFile(engine_type), callback, callback_param1, callback_param2),
 	self_scope(*this, engine_type, v, info_view),
-	parent_scope(*this, engine_type, ((v != NULL) ? v->First() : v), info_view),
+	parent_scope(*this, engine_type, ((v != nullptr) ? v->First() : v), info_view),
 	relative_scope(*this, engine_type, v, info_view),
 	cached_relative_count(0)
 {
 	if (wagon_override == WO_SELF) {
 		this->root_spritegroup = GetWagonOverrideSpriteSet(engine_type, CT_DEFAULT, engine_type);
 	} else {
-		if (wagon_override != WO_NONE && v != NULL && v->IsGroundVehicle()) {
+		if (wagon_override != WO_NONE && v != nullptr && v->IsGroundVehicle()) {
 			assert(v->engine_type == engine_type); // overrides make little sense with fake scopes
 
 			/* For trains we always use cached value, except for callbacks because the override spriteset
@@ -982,11 +1007,11 @@ VehicleResolverObject::VehicleResolverObject(EngineID engine_type, const Vehicle
 			}
 		}
 
-		if (this->root_spritegroup == NULL) {
+		if (this->root_spritegroup == nullptr) {
 			const Engine *e = Engine::Get(engine_type);
-			CargoID cargo = v != NULL ? v->cargo_type : CT_PURCHASE;
+			CargoID cargo = v != nullptr ? v->cargo_type : CT_PURCHASE;
 			assert(cargo < lengthof(e->grf_prop.spritegroup));
-			this->root_spritegroup = e->grf_prop.spritegroup[cargo] != NULL ? e->grf_prop.spritegroup[cargo] : e->grf_prop.spritegroup[CT_DEFAULT];
+			this->root_spritegroup = e->grf_prop.spritegroup[cargo] != nullptr ? e->grf_prop.spritegroup[cargo] : e->grf_prop.spritegroup[CT_DEFAULT];
 		}
 	}
 }
@@ -1005,7 +1030,7 @@ void GetCustomEngineSprite(EngineID engine, const Vehicle *v, Direction directio
 		object.callback_param1 = image_type | (stack << 8);
 		const SpriteGroup *group = object.Resolve();
 		uint32 reg100 = sprite_stack ? GetRegister(0x100) : 0;
-		if (group != NULL && group->GetNumResults() != 0) {
+		if (group != nullptr && group->GetNumResults() != 0) {
 			result->seq[result->count].sprite = group->GetResult() + (direction % group->GetNumResults());
 			result->seq[result->count].pal    = GB(reg100, 0, 16); // zero means default recolouring
 			result->count++;
@@ -1025,7 +1050,7 @@ void GetRotorOverrideSprite(EngineID engine, const struct Aircraft *v, bool info
 
 	VehicleResolverObject object(engine, v, VehicleResolverObject::WO_SELF, info_view, CBID_NO_CALLBACK);
 	result->Clear();
-	uint rotor_pos = v == NULL || info_view ? 0 : v->Next()->Next()->state;
+	uint rotor_pos = v == nullptr || info_view ? 0 : v->Next()->Next()->state;
 
 	bool sprite_stack = HasBit(e->info.misc_flags, EF_SPRITE_STACK);
 	uint max_stack = sprite_stack ? lengthof(result->seq) : 1;
@@ -1034,7 +1059,7 @@ void GetRotorOverrideSprite(EngineID engine, const struct Aircraft *v, bool info
 		object.callback_param1 = image_type | (stack << 8);
 		const SpriteGroup *group = object.Resolve();
 		uint32 reg100 = sprite_stack ? GetRegister(0x100) : 0;
-		if (group != NULL && group->GetNumResults() != 0) {
+		if (group != nullptr && group->GetNumResults() != 0) {
 			result->seq[result->count].sprite = group->GetResult() + (rotor_pos % group->GetNumResults());
 			result->seq[result->count].pal    = GB(reg100, 0, 16); // zero means default recolouring
 			result->count++;
@@ -1052,7 +1077,7 @@ void GetRotorOverrideSprite(EngineID engine, const struct Aircraft *v, bool info
 bool UsesWagonOverride(const Vehicle *v)
 {
 	assert(v->type == VEH_TRAIN);
-	return Train::From(v)->tcache.cached_override != NULL;
+	return Train::From(v)->tcache.cached_override != nullptr;
 }
 
 /**
@@ -1061,7 +1086,7 @@ bool UsesWagonOverride(const Vehicle *v)
  * @param param1   First parameter of the callback
  * @param param2   Second parameter of the callback
  * @param engine   Engine type of the vehicle to evaluate the callback for
- * @param v        The vehicle to evaluate the callback for, or NULL if it doesnt exist yet
+ * @param v        The vehicle to evaluate the callback for, or nullptr if it doesn't exist yet
  * @return The value the callback returned, or CALLBACK_FAILED if it failed
  */
 uint16 GetVehicleCallback(CallbackID callback, uint32 param1, uint32 param2, EngineID engine, const Vehicle *v)
@@ -1076,7 +1101,7 @@ uint16 GetVehicleCallback(CallbackID callback, uint32 param1, uint32 param2, Eng
  * @param param1   First parameter of the callback
  * @param param2   Second parameter of the callback
  * @param engine   Engine type of the vehicle to evaluate the callback for
- * @param v        The vehicle to evaluate the callback for, or NULL if it doesn't exist yet
+ * @param v        The vehicle to evaluate the callback for, or nullptr if it doesn't exist yet
  * @param parent   The vehicle to use for parent scope
  * @return The value the callback returned, or CALLBACK_FAILED if it failed
  */
@@ -1107,14 +1132,14 @@ uint GetEngineProperty(EngineID engine, PropertyID property, uint orig_value, co
 static void DoTriggerVehicle(Vehicle *v, VehicleTrigger trigger, byte base_random_bits, bool first)
 {
 	/* We can't trigger a non-existent vehicle... */
-	assert(v != NULL);
+	assert(v != nullptr);
 
 	VehicleResolverObject object(v->engine_type, v, VehicleResolverObject::WO_CACHED, false, CBID_RANDOM_TRIGGER);
 	object.waiting_triggers = v->waiting_triggers | trigger;
 	v->waiting_triggers = object.waiting_triggers; // store now for var 5F
 
 	const SpriteGroup *group = object.Resolve();
-	if (group == NULL) return;
+	if (group == nullptr) return;
 
 	/* Store remaining triggers. */
 	v->waiting_triggers = object.GetRemainingTriggers();
@@ -1142,7 +1167,7 @@ static void DoTriggerVehicle(Vehicle *v, VehicleTrigger trigger, byte base_rando
 			/* We now trigger the next vehicle in chain recursively.
 			 * The random bits portions may be different for each
 			 * vehicle in chain. */
-			if (v->Next() != NULL) DoTriggerVehicle(v->Next(), trigger, 0, true);
+			if (v->Next() != nullptr) DoTriggerVehicle(v->Next(), trigger, 0, true);
 			break;
 
 		case VEHICLE_TRIGGER_EMPTY:
@@ -1150,14 +1175,14 @@ static void DoTriggerVehicle(Vehicle *v, VehicleTrigger trigger, byte base_rando
 			 * recursively.  The random bits portions must be same
 			 * for each vehicle in chain, so we give them all
 			 * first chained vehicle's portion of random bits. */
-			if (v->Next() != NULL) DoTriggerVehicle(v->Next(), trigger, first ? new_random_bits : base_random_bits, false);
+			if (v->Next() != nullptr) DoTriggerVehicle(v->Next(), trigger, first ? new_random_bits : base_random_bits, false);
 			break;
 
 		case VEHICLE_TRIGGER_ANY_NEW_CARGO:
 			/* Now pass the trigger recursively to the next vehicle
 			 * in chain. */
 			assert(!first);
-			if (v->Next() != NULL) DoTriggerVehicle(v->Next(), VEHICLE_TRIGGER_ANY_NEW_CARGO, base_random_bits, false);
+			if (v->Next() != nullptr) DoTriggerVehicle(v->Next(), VEHICLE_TRIGGER_ANY_NEW_CARGO, base_random_bits, false);
 			break;
 
 		case VEHICLE_TRIGGER_CALLBACK_32:
@@ -1185,7 +1210,7 @@ struct ListOrderChange {
 	uint target;      ///< local ID
 };
 
-static SmallVector<ListOrderChange, 16> _list_order_changes;
+static std::vector<ListOrderChange> _list_order_changes;
 
 /**
  * Record a vehicle ListOrderChange.
@@ -1196,9 +1221,7 @@ static SmallVector<ListOrderChange, 16> _list_order_changes;
 void AlterVehicleListOrder(EngineID engine, uint target)
 {
 	/* Add the list order change to a queue */
-	ListOrderChange *loc = _list_order_changes.Append();
-	loc->engine = engine;
-	loc->target = target;
+	_list_order_changes.push_back({engine, target});
 }
 
 /**
@@ -1207,19 +1230,19 @@ void AlterVehicleListOrder(EngineID engine, uint target)
  * @param b right side
  * @return comparison result
  */
-static int CDECL EnginePreSort(const EngineID *a, const EngineID *b)
+static bool EnginePreSort(const EngineID &a, const EngineID &b)
 {
-	const EngineIDMapping *id_a = _engine_mngr.Get(*a);
-	const EngineIDMapping *id_b = _engine_mngr.Get(*b);
+	const EngineIDMapping &id_a = _engine_mngr.at(a);
+	const EngineIDMapping &id_b = _engine_mngr.at(b);
 
 	/* 1. Sort by engine type */
-	if (id_a->type != id_b->type) return (int)id_a->type - (int)id_b->type;
+	if (id_a.type != id_b.type) return (int)id_a.type < (int)id_b.type;
 
 	/* 2. Sort by scope-GRFID */
-	if (id_a->grfid != id_b->grfid) return id_a->grfid < id_b->grfid ? -1 : 1;
+	if (id_a.grfid != id_b.grfid) return id_a.grfid < id_b.grfid;
 
 	/* 3. Sort by local ID */
-	return (int)id_a->internal_id - (int)id_b->internal_id;
+	return (int)id_a.internal_id < (int)id_b.internal_id;
 }
 
 /**
@@ -1228,32 +1251,30 @@ static int CDECL EnginePreSort(const EngineID *a, const EngineID *b)
 void CommitVehicleListOrderChanges()
 {
 	/* Pre-sort engines by scope-grfid and local index */
-	SmallVector<EngineID, 16> ordering;
-	Engine *e;
-	FOR_ALL_ENGINES(e) {
-		*ordering.Append() = e->index;
+	std::vector<EngineID> ordering;
+	for (const Engine *e : Engine::Iterate()) {
+		ordering.push_back(e->index);
 	}
-	QSortT(ordering.Begin(), ordering.Length(), EnginePreSort);
+	std::sort(ordering.begin(), ordering.end(), EnginePreSort);
 
 	/* Apply Insertion-Sort operations */
-	const ListOrderChange *end = _list_order_changes.End();
-	for (const ListOrderChange *it = _list_order_changes.Begin(); it != end; ++it) {
-		EngineID source = it->engine;
-		uint local_target = it->target;
+	for (const ListOrderChange &it : _list_order_changes) {
+		EngineID source = it.engine;
+		uint local_target = it.target;
 
-		const EngineIDMapping *id_source = _engine_mngr.Get(source);
+		const EngineIDMapping *id_source = _engine_mngr.data() + source;
 		if (id_source->internal_id == local_target) continue;
 
 		EngineID target = _engine_mngr.GetID(id_source->type, local_target, id_source->grfid);
 		if (target == INVALID_ENGINE) continue;
 
-		int source_index = ordering.FindIndex(source);
-		int target_index = ordering.FindIndex(target);
+		int source_index = find_index(ordering, source);
+		int target_index = find_index(ordering, target);
 
 		assert(source_index >= 0 && target_index >= 0);
 		assert(source_index != target_index);
 
-		EngineID *list = ordering.Begin();
+		EngineID *list = ordering.data();
 		if (source_index < target_index) {
 			--target_index;
 			for (int i = source_index; i < target_index; ++i) list[i] = list[i + 1];
@@ -1265,14 +1286,15 @@ void CommitVehicleListOrderChanges()
 	}
 
 	/* Store final sort-order */
-	const EngineID *idend = ordering.End();
 	uint index = 0;
-	for (const EngineID *it = ordering.Begin(); it != idend; ++it, ++index) {
-		Engine::Get(*it)->list_position = index;
+	for (const EngineID &eid : ordering) {
+		Engine::Get(eid)->list_position = index;
+		++index;
 	}
 
 	/* Clear out the queue */
-	_list_order_changes.Reset();
+	_list_order_changes.clear();
+	_list_order_changes.shrink_to_fit();
 }
 
 /**
