@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -31,6 +29,8 @@ struct GenericScopeResolver : public ScopeResolver {
 	uint8 count;
 	uint8 station_size;
 
+	uint8 feature;
+
 	/**
 	 * Generic scope resolver.
 	 * @param ro Surrounding resolver.
@@ -38,11 +38,11 @@ struct GenericScopeResolver : public ScopeResolver {
 	 */
 	GenericScopeResolver(ResolverObject &ro, bool ai_callback)
 		: ScopeResolver(ro), cargo_type(0), default_selection(0), src_industry(0), dst_industry(0), distance(0),
-		event(), count(0), station_size(0), ai_callback(ai_callback)
+		event(), count(0), station_size(0), feature(GSF_INVALID), ai_callback(ai_callback)
 	{
 	}
 
-	/* virtual */ uint32 GetVariable(byte variable, uint32 parameter, bool *available) const;
+	uint32 GetVariable(byte variable, uint32 parameter, bool *available) const override;
 
 private:
 	bool ai_callback; ///< Callback comes from the AI.
@@ -55,7 +55,7 @@ struct GenericResolverObject : public ResolverObject {
 
 	GenericResolverObject(bool ai_callback, CallbackID callback = CBID_NO_CALLBACK);
 
-	/* virtual */ ScopeResolver *GetScope(VarSpriteGroupScope scope = VSG_SCOPE_SELF, byte relative = 0)
+	ScopeResolver *GetScope(VarSpriteGroupScope scope = VSG_SCOPE_SELF, byte relative = 0) override
 	{
 		switch (scope) {
 			case VSG_SCOPE_SELF: return &this->generic_scope;
@@ -63,7 +63,17 @@ struct GenericResolverObject : public ResolverObject {
 		}
 	}
 
-	/* virtual */ const SpriteGroup *ResolveReal(const RealSpriteGroup *group) const;
+	const SpriteGroup *ResolveReal(const RealSpriteGroup *group) const override;
+
+	GrfSpecFeature GetFeature() const override
+	{
+		return (GrfSpecFeature)this->generic_scope.feature;
+	}
+
+	uint32 GetDebugID() const override
+	{
+		return 0;
+	}
 };
 
 struct GenericCallback {
@@ -140,7 +150,7 @@ void AddGenericCallback(uint8 feature, const GRFFile *file, const SpriteGroup *g
 
 /* virtual */ const SpriteGroup *GenericResolverObject::ResolveReal(const RealSpriteGroup *group) const
 {
-	if (group->num_loaded == 0) return NULL;
+	if (group->num_loaded == 0) return nullptr;
 
 	return group->loaded[0];
 }
@@ -150,7 +160,7 @@ void AddGenericCallback(uint8 feature, const GRFFile *file, const SpriteGroup *g
  * @param ai_callback Callback comes from the AI.
  * @param callback Callback ID.
  */
-GenericResolverObject::GenericResolverObject(bool ai_callback, CallbackID callback) : ResolverObject(NULL, callback), generic_scope(*this, ai_callback)
+GenericResolverObject::GenericResolverObject(bool ai_callback, CallbackID callback) : ResolverObject(nullptr, callback), generic_scope(*this, ai_callback)
 {
 }
 
@@ -162,7 +172,7 @@ GenericResolverObject::GenericResolverObject(bool ai_callback, CallbackID callba
  * @param object  pre-populated resolver object
  * @param param1_grfv7 callback_param1 for GRFs up to version 7.
  * @param param1_grfv8 callback_param1 for GRFs from version 8 on.
- * @param[out] file Optionally returns the GRFFile which made the final decision for the callback result. May be NULL if not required.
+ * @param[out] file Optionally returns the GRFFile which made the final decision for the callback result. May be nullptr if not required.
  * @return callback value if successful or CALLBACK_FAILED
  */
 static uint16 GetGenericCallbackResult(uint8 feature, ResolverObject &object, uint32 param1_grfv7, uint32 param1_grfv8, const GRFFile **file)
@@ -179,7 +189,7 @@ static uint16 GetGenericCallbackResult(uint8 feature, ResolverObject &object, ui
 		if (result == CALLBACK_FAILED) continue;
 
 		/* Return NewGRF file if necessary */
-		if (file != NULL) *file = it->file;
+		if (file != nullptr) *file = it->file;
 
 		return result;
 	}
@@ -201,7 +211,7 @@ static uint16 GetGenericCallbackResult(uint8 feature, ResolverObject &object, ui
  * @param event 'AI construction event' to pass to callback. (Variable 86)
  * @param count 'Construction number' to pass to callback. (Variable 87)
  * @param station_size 'Station size' to pass to callback. (Variable 88)
- * @param[out] file Optionally returns the GRFFile which made the final decision for the callback result. May be NULL if not required.
+ * @param[out] file Optionally returns the GRFFile which made the final decision for the callback result. May be nullptr if not required.
  * @return callback value if successful or CALLBACK_FAILED
  */
 uint16 GetAiPurchaseCallbackResult(uint8 feature, CargoID cargo_type, uint8 default_selection, IndustryType src_industry, IndustryType dst_industry, uint8 distance, AIConstructionEvent event, uint8 count, uint8 station_size, const GRFFile **file)
@@ -228,6 +238,7 @@ uint16 GetAiPurchaseCallbackResult(uint8 feature, CargoID cargo_type, uint8 defa
 	object.generic_scope.event             = event;
 	object.generic_scope.count             = count;
 	object.generic_scope.station_size      = station_size;
+	object.generic_scope.feature           = feature;
 
 	uint16 callback = GetGenericCallbackResult(feature, object, 0, 0, file);
 	if (callback != CALLBACK_FAILED) callback = GB(callback, 0, 8);
@@ -249,6 +260,7 @@ void AmbientSoundEffectCallback(TileIndex tile)
 
 	/* Prepare resolver object. */
 	GenericResolverObject object(false, CBID_SOUNDS_AMBIENT_EFFECT);
+	object.generic_scope.feature = GSF_SOUNDFX;
 
 	uint32 param1_v7 = GetTileType(tile) << 28 | Clamp(TileHeight(tile), 0, 15) << 24 | GB(r, 16, 8) << 16 | GetTerrainType(tile);
 	uint32 param1_v8 = GetTileType(tile) << 24 | GetTileZ(tile) << 16 | GB(r, 16, 8) << 8 | (HasTileWaterClass(tile) ? GetWaterClass(tile) : 0) << 3 | GetTerrainType(tile);
