@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -15,9 +13,13 @@
 #include "road_type.h"
 #include "tile_type.h"
 #include "direction_type.h"
+#include "widgets/dropdown_type.h"
 
 struct Window *ShowBuildRoadToolbar(RoadType roadtype);
-struct Window *ShowBuildRoadScenToolbar();
+struct Window *ShowBuildRoadScenToolbar(RoadType roadtype);
 void ConnectRoadToStructure(TileIndex tile, DiagDirection direction);
+DropDownList GetRoadTypeDropDownList(RoadTramTypes rtts, bool for_replacement = false, bool all_option = false);
+DropDownList GetScenRoadTypeDropDownList(RoadTramTypes rtts);
+void InitializeRoadGUI();
 
 #endif /* ROAD_GUI_H */
