@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -32,9 +30,7 @@ static const SaveLoad _depot_desc[] = {
 
 static void Save_DEPT()
 {
-	Depot *depot;
-
-	FOR_ALL_DEPOTS(depot) {
+	for (Depot *depot : Depot::Iterate()) {
 		SlSetArrayIndex(depot->index);
 		SlObject(depot, _depot_desc);
 	}
@@ -55,14 +51,12 @@ static void Load_DEPT()
 
 static void Ptrs_DEPT()
 {
-	Depot *depot;
-
-	FOR_ALL_DEPOTS(depot) {
+	for (Depot *depot : Depot::Iterate()) {
 		SlObject(depot, _depot_desc);
 		if (IsSavegameVersionBefore(SLV_141)) depot->town = Town::Get((size_t)depot->town);
 	}
 }
 
 extern const ChunkHandler _depot_chunk_handlers[] = {
-	{ 'DEPT', Save_DEPT, Load_DEPT, Ptrs_DEPT, NULL, CH_ARRAY | CH_LAST},
+	{ 'DEPT', Save_DEPT, Load_DEPT, Ptrs_DEPT, nullptr, CH_ARRAY | CH_LAST},
 };
