@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -106,7 +104,7 @@ assert_compile(lengthof(_glog_desc) == GLCT_END);
 
 static void Load_GLOG_common(LoggedAction *&gamelog_action, uint &gamelog_actions)
 {
-	assert(gamelog_action == NULL);
+	assert(gamelog_action == nullptr);
 	assert(gamelog_actions == 0);
 
 	GamelogActionType at;
@@ -117,7 +115,7 @@ static void Load_GLOG_common(LoggedAction *&gamelog_action, uint &gamelog_action
 		la->at = at;
 
 		SlObject(la, _glog_action_desc); // has to be saved after 'DATE'!
-		la->change = NULL;
+		la->change = nullptr;
 		la->changes = 0;
 
 		GamelogChangeType ct;
@@ -125,7 +123,7 @@ static void Load_GLOG_common(LoggedAction *&gamelog_action, uint &gamelog_action
 			la->change = ReallocT(la->change, la->changes + 1);
 
 			LoggedChange *lc = &la->change[la->changes++];
-			/* for SLE_STR, pointer has to be valid! so make it NULL */
+			/* for SLE_STR, pointer has to be valid! so make it nullptr */
 			memset(lc, 0, sizeof(*lc));
 			lc->ct = ct;
 
@@ -179,5 +177,5 @@ static void Check_GLOG()
 }
 
 extern const ChunkHandler _gamelog_chunk_handlers[] = {
-	{ 'GLOG', Save_GLOG, Load_GLOG, NULL, Check_GLOG, CH_RIFF | CH_LAST }
+	{ 'GLOG', Save_GLOG, Load_GLOG, nullptr, Check_GLOG, CH_RIFF | CH_LAST }
 };
