@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -31,6 +29,7 @@
 #include "../core/smallvec_type.hpp"
 #include "saveload_internal.h"
 #include "oldloader.h"
+#include <array>
 
 #include "table/strings.h"
 #include "../table/engines.h"
@@ -110,8 +109,7 @@ static void FixTTDMapArray()
 
 static void FixTTDDepots()
 {
-	const Depot *d;
-	FOR_ALL_DEPOTS_FROM(d, 252) {
+	for (const Depot *d : Depot::Iterate(252)) {
 		if (!IsDepotTile(d->xy) || GetDepotIndex(d->xy) != d->index) {
 			/** Workaround for SVXConverter bug, depots 252-255 could be invalid */
 			delete d;
@@ -154,10 +152,8 @@ static uint32 RemapOldTownName(uint32 townnameparts, byte old_town_name_type)
 
 static void FixOldTowns()
 {
-	Town *town;
-
 	/* Convert town-names if needed */
-	FOR_ALL_TOWNS(town) {
+	for (Town *town : Town::Iterate()) {
 		if (IsInsideMM(town->townnametype, 0x20C1, 0x20C3)) {
 			town->townnametype = SPECSTR_TOWNNAME_ENGLISH + _settings_game.game_creation.town_name;
 			town->townnameparts = RemapOldTownName(town->townnameparts, _settings_game.game_creation.town_name);
@@ -174,11 +170,9 @@ static StringID *_old_vehicle_names;
  */
 void FixOldVehicles()
 {
-	Vehicle *v;
-
-	FOR_ALL_VEHICLES(v) {
+	for (Vehicle *v : Vehicle::Iterate()) {
 		if ((size_t)v->next == 0xFFFF) {
-			v->next = NULL;
+			v->next = nullptr;
 		} else {
 			v->next = Vehicle::GetIfValid((size_t)v->next);
 		}
@@ -386,8 +380,7 @@ static bool FixTTOEngines()
 		233, 234, 235, 236, 237, 238, 253
 	};
 
-	Vehicle *v;
-	FOR_ALL_VEHICLES(v) {
+	for (Vehicle *v : Vehicle::Iterate()) {
 		if (v->engine_type >= lengthof(tto_to_ttd)) return false;
 		v->engine_type = tto_to_ttd[v->engine_type];
 	}
@@ -452,7 +445,7 @@ static bool FixTTOEngines()
 		e->preview_company = INVALID_COMPANY;
 		e->preview_asked = (CompanyMask)-1;
 		e->preview_wait = 0;
-		e->name = NULL;
+		e->name = nullptr;
 	}
 
 	return true;
@@ -460,8 +453,7 @@ static bool FixTTOEngines()
 
 static void FixTTOCompanies()
 {
-	Company *c;
-	FOR_ALL_COMPANIES(c) {
+	for (Company *c : Company::Iterate()) {
 		c->cur_economy.company_value = CalculateCompanyValue(c); // company value history is zeroed
 	}
 }
@@ -491,7 +483,7 @@ static inline uint RemapOrderIndex(uint x)
 	return _savegame_type == SGT_TTO ? (x - 0x1AC4) / 2 : (x - 0x1C18) / 2;
 }
 
-extern SmallVector<TileIndex, 256> _animated_tiles;
+extern std::vector<TileIndex> _animated_tiles;
 extern char *_old_name_array;
 
 static uint32 _old_town_index;
@@ -621,7 +613,7 @@ static const OldChunks order_chunk[] = {
 
 static bool LoadOldOrder(LoadgameState *ls, int num)
 {
-	if (!LoadChunk(ls, NULL, order_chunk)) return false;
+	if (!LoadChunk(ls, nullptr, order_chunk)) return false;
 
 	Order *o = new (num) Order();
 	o->AssignOrder(UnpackOldOrder(_old_order));
@@ -632,7 +624,7 @@ static bool LoadOldOrder(LoadgameState *ls, int num)
 		/* Relink the orders to each other (in the orders for one vehicle are behind each other,
 		 * with an invalid order (OT_NOTHING) as indication that it is the last order */
 		Order *prev = Order::GetIfValid(num - 1);
-		if (prev != NULL) prev->next = o;
+		if (prev != nullptr) prev->next = o;
 	}
 
 	return true;
@@ -646,12 +638,12 @@ static bool LoadOldAnimTileList(LoadgameState *ls, int num)
 		OCL_END ()
 	};
 
-	if (!LoadChunk(ls, NULL, anim_chunk)) return false;
+	if (!LoadChunk(ls, nullptr, anim_chunk)) return false;
 
 	/* The first zero in the loaded array indicates the end of the list. */
 	for (int i = 0; i < 256; i++) {
 		if (anim_list[i] == 0) break;
-		*_animated_tiles.Append() = anim_list[i];
+		_animated_tiles.push_back(anim_list[i]);
 	}
 
 	return true;
@@ -671,7 +663,7 @@ static bool LoadOldDepot(LoadgameState *ls, int num)
 	if (d->xy != 0) {
 		/* In some cases, there could be depots referencing invalid town. */
 		Town *t = Town::GetIfValid(RemapTownIndex(_old_town_index));
-		if (t == NULL) t = Town::GetRandom();
+		if (t == nullptr) t = Town::GetRandom();
 		d->town = t;
 	} else {
 		delete d;
@@ -724,7 +716,7 @@ static const OldChunks station_chunk[] = {
 	OCL_NULL( 4 ), ///< bus/lorry tile
 	OCL_SVAR(   OC_TILE, Station, train_station.tile ),
 	OCL_SVAR(   OC_TILE, Station, airport.tile ),
-	OCL_SVAR(   OC_TILE, Station, dock_tile ),
+	OCL_NULL( 2 ), ///< dock tile
 	OCL_SVAR( OC_FILE_U8 | OC_VAR_U16, Station, train_station.w ),
 
 	OCL_NULL( 1 ),         ///< sort-index, no longer in use
@@ -876,7 +868,7 @@ static bool LoadOldCompanyYearly(LoadgameState *ls, int num)
 		if (_savegame_type == SGT_TTO && i == 6) {
 			_old_yearly = 0; // property maintenance
 		} else {
-			if (!LoadChunk(ls, NULL, _company_yearly_chunk)) return false;
+			if (!LoadChunk(ls, nullptr, _company_yearly_chunk)) return false;
 		}
 
 		c->yearly_expenses[num][i] = _old_yearly;
@@ -1106,8 +1098,8 @@ static bool LoadOldVehicleUnion(LoadgameState *ls, int num)
 	uint temp = ls->total_read;
 	bool res;
 
-	if (v == NULL) {
-		res = LoadChunk(ls, NULL, vehicle_empty_chunk);
+	if (v == nullptr) {
+		res = LoadChunk(ls, nullptr, vehicle_empty_chunk);
 	} else {
 		switch (v->type) {
 			default: SlErrorCorrupt("Invalid vehicle type");
@@ -1240,7 +1232,7 @@ bool LoadOldVehicle(LoadgameState *ls, int num)
 			uint type = ReadByte(ls);
 			switch (type) {
 				default: return false;
-				case 0x00 /* VEH_INVALID  */: v = NULL;                                        break;
+				case 0x00 /* VEH_INVALID  */: v = nullptr;                                        break;
 				case 0x25 /* MONORAIL     */:
 				case 0x20 /* VEH_TRAIN    */: v = new (_current_vehicle_id) Train();           break;
 				case 0x21 /* VEH_ROAD     */: v = new (_current_vehicle_id) RoadVehicle();     break;
@@ -1251,7 +1243,7 @@ bool LoadOldVehicle(LoadgameState *ls, int num)
 			}
 
 			if (!LoadChunk(ls, v, vehicle_chunk)) return false;
-			if (v == NULL) continue;
+			if (v == nullptr) continue;
 			v->refit_cap = v->cargo_cap;
 
 			SpriteID sprite = v->sprite_seq.seq[0].sprite;
@@ -1276,7 +1268,8 @@ bool LoadOldVehicle(LoadgameState *ls, int num)
 					};
 					if (v->spritenum / 2 >= lengthof(spriteset_rail)) return false;
 					v->spritenum = spriteset_rail[v->spritenum / 2]; // adjust railway sprite set offset
-					Train::From(v)->railtype = type == 0x25 ? 1 : 0; // monorail / rail
+					/* Should be the original values for monorail / rail, can't use RailType constants */
+					Train::From(v)->railtype = static_cast<RailType>(type == 0x25 ? 1 : 0);
 					break;
 				}
 
@@ -1318,7 +1311,7 @@ bool LoadOldVehicle(LoadgameState *ls, int num)
 			/* Read the vehicle type and allocate the right vehicle */
 			switch (ReadByte(ls)) {
 				default: SlErrorCorrupt("Invalid vehicle type");
-				case 0x00 /* VEH_INVALID */: v = NULL;                                        break;
+				case 0x00 /* VEH_INVALID */: v = nullptr;                                        break;
 				case 0x10 /* VEH_TRAIN   */: v = new (_current_vehicle_id) Train();           break;
 				case 0x11 /* VEH_ROAD    */: v = new (_current_vehicle_id) RoadVehicle();     break;
 				case 0x12 /* VEH_SHIP    */: v = new (_current_vehicle_id) Ship();            break;
@@ -1328,7 +1321,7 @@ bool LoadOldVehicle(LoadgameState *ls, int num)
 			}
 
 			if (!LoadChunk(ls, v, vehicle_chunk)) return false;
-			if (v == NULL) continue;
+			if (v == nullptr) continue;
 
 			_old_vehicle_names[_current_vehicle_id] = RemapOldStringID(_old_string_id);
 
@@ -1754,11 +1747,11 @@ bool LoadTTDMain(LoadgameState *ls)
 	_read_ttdpatch_flags = false;
 
 	/* Load the biggest chunk */
-	SmallStackSafeStackAlloc<byte, OLD_MAP_SIZE * 2> map3;
-	_old_map3 = map3.data;
-	_old_vehicle_names = NULL;
+	std::array<byte, OLD_MAP_SIZE * 2> map3;
+	_old_map3 = map3.data();
+	_old_vehicle_names = nullptr;
 	try {
-		if (!LoadChunk(ls, NULL, main_chunk)) {
+		if (!LoadChunk(ls, nullptr, main_chunk)) {
 			DEBUG(oldloader, 0, "Loading failed");
 			free(_old_vehicle_names);
 			return false;
@@ -1797,13 +1790,13 @@ bool LoadTTOMain(LoadgameState *ls)
 
 	_read_ttdpatch_flags = false;
 
-	SmallStackSafeStackAlloc<byte, 103 * sizeof(Engine)> engines; // we don't want to call Engine constructor here
-	_old_engines = (Engine *)engines.data;
-	SmallStackSafeStackAlloc<StringID, 800> vehnames;
-	_old_vehicle_names = vehnames.data;
+	std::array<byte, 103 * sizeof(Engine)> engines; // we don't want to call Engine constructor here
+	_old_engines = (Engine *)engines.data();
+	std::array<StringID, 800> vehnames;
+	_old_vehicle_names = vehnames.data();
 
 	/* Load the biggest chunk */
-	if (!LoadChunk(ls, NULL, main_chunk)) {
+	if (!LoadChunk(ls, nullptr, main_chunk)) {
 		DEBUG(oldloader, 0, "Loading failed");
 		return false;
 	}
