@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -39,9 +37,5 @@
 
 /* static */ bool ScriptGame::IsMultiplayer()
 {
-#ifdef ENABLE_NETWORK
 	return _network_server;
-#else
-	return false;
-#endif
 }
