@@ -1,5 +1,3 @@
-/* $Id: script_list.cpp 27594 2016-05-30 21:03:11Z frosch $ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -109,7 +107,7 @@ public:
 
 	void End()
 	{
-		this->bucket_list = NULL;
+		this->bucket_list = nullptr;
 		this->has_no_more_items = true;
 		this->item_next = 0;
 	}
@@ -119,7 +117,7 @@ public:
 	 */
 	void FindNext()
 	{
-		if (this->bucket_list == NULL) {
+		if (this->bucket_list == nullptr) {
 			this->has_no_more_items = true;
 			return;
 		}
@@ -128,7 +126,7 @@ public:
 		if (this->bucket_list_iter == this->bucket_list->end()) {
 			this->bucket_iter++;
 			if (this->bucket_iter == this->list->buckets.end()) {
-				this->bucket_list = NULL;
+				this->bucket_list = nullptr;
 				return;
 			}
 			this->bucket_list = &(*this->bucket_iter).second;
@@ -203,7 +201,7 @@ public:
 
 	void End()
 	{
-		this->bucket_list = NULL;
+		this->bucket_list = nullptr;
 		this->has_no_more_items = true;
 		this->item_next = 0;
 	}
@@ -213,14 +211,14 @@ public:
 	 */
 	void FindNext()
 	{
-		if (this->bucket_list == NULL) {
+		if (this->bucket_list == nullptr) {
 			this->has_no_more_items = true;
 			return;
 		}
 
 		if (this->bucket_list_iter == this->bucket_list->begin()) {
 			if (this->bucket_iter == this->list->buckets.begin()) {
-				this->bucket_list = NULL;
+				this->bucket_list = nullptr;
 				return;
 			}
 			this->bucket_iter--;
@@ -845,7 +843,7 @@ SQInteger ScriptList::Valuate(HSQUIRRELVM vm)
 	int nparam = sq_gettop(vm) - 1;
 
 	if (nparam < 1) {
-		return sq_throwerror(vm, "You need to give a least a Valuator as parameter to ScriptList::Valuate");
+		return sq_throwerror(vm, "You need to give at least a Valuator as parameter to ScriptList::Valuate");
 	}
 
 	/* Make sure the valuator function is really a function, and not any
