@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -33,12 +31,12 @@
 		case MP_WATER: return IsCoast(tile);
 		case MP_ROAD:
 			/* Tram bits aren't considered buildable */
-			if (::GetRoadTypes(tile) != ROADTYPES_ROAD) return false;
+			if (::GetRoadTypeTram(tile) != INVALID_ROADTYPE) return false;
 			/* Depots and crossings aren't considered buildable */
 			if (::GetRoadTileType(tile) != ROAD_TILE_NORMAL) return false;
-			if (!HasExactlyOneBit(::GetRoadBits(tile, ROADTYPE_ROAD))) return false;
-			if (::IsRoadOwner(tile, ROADTYPE_ROAD, OWNER_TOWN)) return true;
-			if (::IsRoadOwner(tile, ROADTYPE_ROAD, ScriptObject::GetCompany())) return true;
+			if (!HasExactlyOneBit(::GetRoadBits(tile, RTT_ROAD))) return false;
+			if (::IsRoadOwner(tile, RTT_ROAD, OWNER_TOWN)) return true;
+			if (::IsRoadOwner(tile, RTT_ROAD, ScriptObject::GetCompany())) return true;
 			return false;
 	}
 }
@@ -201,7 +199,12 @@
 {
 	if (!::IsValidTile(tile)) return false;
 
-	return ::TrackStatusToTrackdirBits(::GetTileTrackStatus(tile, (::TransportType)transport_type, UINT32_MAX)) != TRACKDIR_BIT_NONE;
+	if (transport_type == TRANSPORT_ROAD) {
+		return ::TrackStatusToTrackdirBits(::GetTileTrackStatus(tile, (::TransportType)transport_type, 0)) != TRACKDIR_BIT_NONE ||
+				::TrackStatusToTrackdirBits(::GetTileTrackStatus(tile, (::TransportType)transport_type, 1)) != TRACKDIR_BIT_NONE;
+	} else {
+		return ::TrackStatusToTrackdirBits(::GetTileTrackStatus(tile, (::TransportType)transport_type, 0)) != TRACKDIR_BIT_NONE;
+	}
 }
 
 /* static */ int32 ScriptTile::GetCargoAcceptance(TileIndex tile, CargoID cargo_type, int width, int height, int radius)
@@ -257,7 +260,6 @@
 
 /* static */ bool ScriptTile::DemolishTile(TileIndex tile)
 {
-	EnforcePrecondition(false, ScriptObject::GetCompany() != OWNER_DEITY);
 	EnforcePrecondition(false, ::IsValidTile(tile));
 
 	return ScriptObject::DoCommand(tile, 0, 0, CMD_LANDSCAPE_CLEAR);
@@ -292,7 +294,7 @@
 	if (!::IsValidTile(tile)) return INVALID_TOWN;
 
 	Town *town = ::ClosestTownFromTile(tile, _settings_game.economy.dist_local_authority);
-	if (town == NULL) return INVALID_TOWN;
+	if (town == nullptr) return INVALID_TOWN;
 
 	return town->index;
 }
@@ -302,7 +304,7 @@
 	if (!::IsValidTile(tile)) return INVALID_TOWN;
 
 	Town *town = ::ClosestTownFromTile(tile, UINT_MAX);
-	if (town == NULL) return INVALID_TOWN;
+	if (town == nullptr) return INVALID_TOWN;
 
 	return town->index;
 }
@@ -310,14 +312,14 @@
 /* static */ Money ScriptTile::GetBuildCost(BuildType build_type)
 {
 	switch (build_type) {
-		case BT_FOUNDATION:   return ::GetPrice(PR_BUILD_FOUNDATION, 1, NULL);
-		case BT_TERRAFORM:    return ::GetPrice(PR_TERRAFORM, 1, NULL);
-		case BT_BUILD_TREES:  return ::GetPrice(PR_BUILD_TREES, 1, NULL);
-		case BT_CLEAR_GRASS:  return ::GetPrice(PR_CLEAR_GRASS, 1, NULL);
-		case BT_CLEAR_ROUGH:  return ::GetPrice(PR_CLEAR_ROUGH, 1, NULL);
-		case BT_CLEAR_ROCKY:  return ::GetPrice(PR_CLEAR_ROCKS, 1, NULL);
-		case BT_CLEAR_FIELDS: return ::GetPrice(PR_CLEAR_FIELDS, 1, NULL);
-		case BT_CLEAR_HOUSE:  return ::GetPrice(PR_CLEAR_HOUSE, 1, NULL);
+		case BT_FOUNDATION:   return ::GetPrice(PR_BUILD_FOUNDATION, 1, nullptr);
+		case BT_TERRAFORM:    return ::GetPrice(PR_TERRAFORM, 1, nullptr);
+		case BT_BUILD_TREES:  return ::GetPrice(PR_BUILD_TREES, 1, nullptr);
+		case BT_CLEAR_GRASS:  return ::GetPrice(PR_CLEAR_GRASS, 1, nullptr);
+		case BT_CLEAR_ROUGH:  return ::GetPrice(PR_CLEAR_ROUGH, 1, nullptr);
+		case BT_CLEAR_ROCKY:  return ::GetPrice(PR_CLEAR_ROCKS, 1, nullptr);
+		case BT_CLEAR_FIELDS: return ::GetPrice(PR_CLEAR_FIELDS, 1, nullptr);
+		case BT_CLEAR_HOUSE:  return ::GetPrice(PR_CLEAR_HOUSE, 1, nullptr);
 		default: return -1;
 	}
 }
