@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -29,10 +27,10 @@ namespace SQConvert {
 	 *  comes out of scope. Useful to make sure you can use stredup(),
 	 *  without leaking memory.
 	 */
-	struct SQAutoFreePointers : SmallVector<void *, 1> {
+	struct SQAutoFreePointers : std::vector<void *> {
 		~SQAutoFreePointers()
 		{
-			for (uint i = 0; i < this->items; i++) free(this->data[i]);
+			for (void * p : *this) free(p);
 		}
 	};
 
@@ -88,8 +86,8 @@ namespace SQConvert {
 	template <> inline int Return<int64>       (HSQUIRRELVM vm, int64 res)       { sq_pushinteger(vm, res); return 1; }
 	template <> inline int Return<Money>       (HSQUIRRELVM vm, Money res)       { sq_pushinteger(vm, res); return 1; }
 	template <> inline int Return<bool>        (HSQUIRRELVM vm, bool res)        { sq_pushbool   (vm, res); return 1; }
-	template <> inline int Return<char *>      (HSQUIRRELVM vm, char *res)       { if (res == NULL) sq_pushnull(vm); else { sq_pushstring(vm, res, -1); free(res); } return 1; }
-	template <> inline int Return<const char *>(HSQUIRRELVM vm, const char *res) { if (res == NULL) sq_pushnull(vm); else { sq_pushstring(vm, res, -1); } return 1; }
+	template <> inline int Return<char *>      (HSQUIRRELVM vm, char *res)       { if (res == nullptr) sq_pushnull(vm); else { sq_pushstring(vm, res, -1); free(res); } return 1; }
+	template <> inline int Return<const char *>(HSQUIRRELVM vm, const char *res) { if (res == nullptr) sq_pushnull(vm); else { sq_pushstring(vm, res, -1); } return 1; }
 	template <> inline int Return<void *>      (HSQUIRRELVM vm, void *res)       { sq_pushuserpointer(vm, res); return 1; }
 	template <> inline int Return<HSQOBJECT>   (HSQUIRRELVM vm, HSQOBJECT res)   { sq_pushobject(vm, res); return 1; }
 
@@ -117,7 +115,7 @@ namespace SQConvert {
 		sq_getstring(vm, -1, &tmp);
 		char *tmp_str = stredup(tmp);
 		sq_poptop(vm);
-		*ptr->Append() = (void *)tmp_str;
+		ptr->push_back((void *)tmp_str);
 		str_validate(tmp_str, tmp_str + strlen(tmp_str));
 		return tmp_str;
 	}
@@ -132,12 +130,12 @@ namespace SQConvert {
 		sq_pushobject(vm, obj);
 		sq_pushnull(vm);
 
-		SmallVector<int32, 2> data;
+		std::vector<int32> data;
 
 		while (SQ_SUCCEEDED(sq_next(vm, -2))) {
 			SQInteger tmp;
 			if (SQ_SUCCEEDED(sq_getinteger(vm, -1, &tmp))) {
-				*data.Append() = (int32)tmp;
+				data.push_back((int32)tmp);
 			} else {
 				sq_pop(vm, 4);
 				throw sq_throwerror(vm, "a member of an array used as parameter to a function is not numeric");
@@ -147,11 +145,11 @@ namespace SQConvert {
 		}
 		sq_pop(vm, 2);
 
-		Array *arr = (Array*)MallocT<byte>(sizeof(Array) + sizeof(int32) * data.Length());
-		arr->size = data.Length();
-		memcpy(arr->array, data.Begin(), sizeof(int32) * data.Length());
+		Array *arr = (Array*)MallocT<byte>(sizeof(Array) + sizeof(int32) * data.size());
+		arr->size = data.size();
+		memcpy(arr->array, data.data(), sizeof(int32) * data.size());
 
-		*ptr->Append() = arr;
+		ptr->push_back(arr);
 		return arr;
 	}
 
@@ -739,8 +737,8 @@ namespace SQConvert {
 	{
 		/* Find the amount of params we got */
 		int nparam = sq_gettop(vm);
-		SQUserPointer ptr = NULL;
-		SQUserPointer real_instance = NULL;
+		SQUserPointer ptr = nullptr;
+		SQUserPointer real_instance = nullptr;
 		HSQOBJECT instance;
 
 		/* Get the 'SQ' instance of this class */
@@ -759,14 +757,14 @@ namespace SQConvert {
 		sq_getinstanceup(vm, 1, &real_instance, 0);
 		/* Get the real function pointer */
 		sq_getuserdata(vm, nparam, &ptr, 0);
-		if (real_instance == NULL) return sq_throwerror(vm, "couldn't detect real instance of class for non-static call");
+		if (real_instance == nullptr) return sq_throwerror(vm, "couldn't detect real instance of class for non-static call");
 		/* Remove the userdata from the stack */
 		sq_pop(vm, 1);
 
 		try {
 			/* Delegate it to a template that can handle this specific function */
 			return HelperT<Tmethod>::SQCall((Tcls *)real_instance, *(Tmethod *)ptr, vm);
-		} catch (SQInteger e) {
+		} catch (SQInteger &e) {
 			return e;
 		}
 	}
@@ -781,8 +779,8 @@ namespace SQConvert {
 	{
 		/* Find the amount of params we got */
 		int nparam = sq_gettop(vm);
-		SQUserPointer ptr = NULL;
-		SQUserPointer real_instance = NULL;
+		SQUserPointer ptr = nullptr;
+		SQUserPointer real_instance = nullptr;
 		HSQOBJECT instance;
 
 		/* Get the 'SQ' instance of this class */
@@ -801,7 +799,7 @@ namespace SQConvert {
 		sq_getinstanceup(vm, 1, &real_instance, 0);
 		/* Get the real function pointer */
 		sq_getuserdata(vm, nparam, &ptr, 0);
-		if (real_instance == NULL) return sq_throwerror(vm, "couldn't detect real instance of class for non-static call");
+		if (real_instance == nullptr) return sq_throwerror(vm, "couldn't detect real instance of class for non-static call");
 		/* Remove the userdata from the stack */
 		sq_pop(vm, 1);
 
@@ -819,15 +817,15 @@ namespace SQConvert {
 	{
 		/* Find the amount of params we got */
 		int nparam = sq_gettop(vm);
-		SQUserPointer ptr = NULL;
+		SQUserPointer ptr = nullptr;
 
 		/* Get the real function pointer */
 		sq_getuserdata(vm, nparam, &ptr, 0);
 
 		try {
 			/* Delegate it to a template that can handle this specific function */
-			return HelperT<Tmethod>::SQCall((Tcls *)NULL, *(Tmethod *)ptr, vm);
-		} catch (SQInteger e) {
+			return HelperT<Tmethod>::SQCall((Tcls *)nullptr, *(Tmethod *)ptr, vm);
+		} catch (SQInteger &e) {
 			return e;
 		}
 	}
@@ -843,7 +841,7 @@ namespace SQConvert {
 	{
 		/* Find the amount of params we got */
 		int nparam = sq_gettop(vm);
-		SQUserPointer ptr = NULL;
+		SQUserPointer ptr = nullptr;
 
 		/* Get the real function pointer */
 		sq_getuserdata(vm, nparam, &ptr, 0);
@@ -862,7 +860,7 @@ namespace SQConvert {
 	static SQInteger DefSQDestructorCallback(SQUserPointer p, SQInteger size)
 	{
 		/* Remove the real instance too */
-		if (p != NULL) ((Tcls *)p)->Release();
+		if (p != nullptr) ((Tcls *)p)->Release();
 		return 0;
 	}
 
@@ -876,12 +874,12 @@ namespace SQConvert {
 	{
 		try {
 			/* Create the real instance */
-			Tcls *instance = HelperT<Tmethod>::SQConstruct((Tcls *)NULL, (Tmethod)NULL, vm);
+			Tcls *instance = HelperT<Tmethod>::SQConstruct((Tcls *)nullptr, (Tmethod)nullptr, vm);
 			sq_setinstanceup(vm, -Tnparam, instance);
 			sq_setreleasehook(vm, -Tnparam, DefSQDestructorCallback<Tcls>);
 			instance->AddRef();
 			return 0;
-		} catch (SQInteger e) {
+		} catch (SQInteger &e) {
 			return e;
 		}
 	}
@@ -903,7 +901,7 @@ namespace SQConvert {
 			sq_setreleasehook(vm, -nparam, DefSQDestructorCallback<Tcls>);
 			instance->AddRef();
 			return 0;
-		} catch (SQInteger e) {
+		} catch (SQInteger &e) {
 			return e;
 		}
 	}
