@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -7,7 +5,7 @@
  * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** @file allegro_s.h Base fo playing sound via Allegro. */
+/** @file allegro_s.h Base for playing sound via Allegro. */
 
 #ifndef SOUND_ALLEGRO_H
 #define SOUND_ALLEGRO_H
@@ -17,12 +15,12 @@
 /** Implementation of the allegro sound driver. */
 class SoundDriver_Allegro : public SoundDriver {
 public:
-	/* virtual */ const char *Start(const char * const *param);
+	const char *Start(const char * const *param);
 
-	/* virtual */ void Stop();
+	void Stop();
 
-	/* virtual */ void MainLoop();
-	/* virtual */ const char *GetName() const { return "allegro"; }
+	void MainLoop();
+	const char *GetName() const { return "allegro"; }
 };
 
 /** Factory for the allegro sound driver. */
