@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -17,17 +15,17 @@
 /** Implementation of the null sound driver. */
 class SoundDriver_Null : public SoundDriver {
 public:
-	/* virtual */ const char *Start(const char * const *param) { return NULL; }
+	const char *Start(const char * const *param) override { return nullptr; }
 
-	/* virtual */ void Stop() { }
-	/* virtual */ const char *GetName() const { return "null"; }
+	void Stop() override { }
+	const char *GetName() const override { return "null"; }
 };
 
 /** Factory for the null sound driver. */
 class FSoundDriver_Null : public DriverFactoryBase {
 public:
 	FSoundDriver_Null() : DriverFactoryBase(Driver::DT_SOUND, 1, "null", "Null Sound Driver") {}
-	/* virtual */ Driver *CreateInstance() const { return new SoundDriver_Null(); }
+	Driver *CreateInstance() const override { return new SoundDriver_Null(); }
 };
 
 #endif /* SOUND_NULL_H */
