@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -29,22 +27,20 @@
 /* Default of 4MB spritecache */
 uint _sprite_cache_size = 4;
 
-typedef SimpleTinyEnumT<SpriteType, byte> SpriteTypeByte;
-
 struct SpriteCache {
 	void *ptr;
 	size_t file_pos;
 	uint32 id;
 	uint16 file_slot;
 	int16 lru;
-	SpriteTypeByte type; ///< In some cases a single sprite is misused by two NewGRFs. Once as real sprite and once as recolour sprite. If the recolour sprite gets into the cache it might be drawn as real sprite which causes enormous trouble.
+	SpriteType type;     ///< In some cases a single sprite is misused by two NewGRFs. Once as real sprite and once as recolour sprite. If the recolour sprite gets into the cache it might be drawn as real sprite which causes enormous trouble.
 	bool warned;         ///< True iff the user has been warned about incorrect use of this sprite
 	byte container_ver;  ///< Container version of the GRF the sprite is from.
 };
 
 
 static uint _spritecache_items = 0;
-static SpriteCache *_spritecache = NULL;
+static SpriteCache *_spritecache = nullptr;
 
 
 static inline SpriteCache *GetSpriteCache(uint index)
@@ -149,6 +145,17 @@ uint GetOriginFileSlot(SpriteID sprite)
 	return GetSpriteCache(sprite)->file_slot;
 }
 
+/**
+ * Get the GRF-local sprite id of a given sprite.
+ * @param sprite The sprite to look at.
+ * @return The GRF-local sprite id.
+ */
+uint32 GetSpriteLocalID(SpriteID sprite)
+{
+	if (!SpriteExists(sprite)) return 0;
+	return GetSpriteCache(sprite)->id;
+}
+
 /**
  * Count the sprites which originate from a specific file slot in a range of SpriteIDs.
  * @param file_slot FIOS file slot.
@@ -424,7 +431,7 @@ static void *ReadSprite(const SpriteCache *sc, SpriteID id, SpriteType sprite_ty
 	}
 
 	if (sprite_avail == 0) {
-		if (sprite_type == ST_MAPGEN) return NULL;
+		if (sprite_type == ST_MAPGEN) return nullptr;
 		if (id == SPR_IMG_QUERY) usererror("Okay... something went horribly wrong. I couldn't load the fallback sprite. What should I do?");
 		return (void*)GetRawSprite(SPR_IMG_QUERY, ST_NORMAL, allocator);
 	}
@@ -535,7 +542,7 @@ bool LoadNextSprite(int load_index, byte file_slot, uint file_sprite_id, byte co
 	byte grf_type = FioReadByte();
 
 	SpriteType type;
-	void *data = NULL;
+	void *data = nullptr;
 	if (grf_type == 0xFF) {
 		/* Some NewGRF files have "empty" pseudo-sprites which are 1
 		 * byte long. Catch these so the sprites won't be displayed. */
@@ -595,7 +602,7 @@ void DupSprite(SpriteID old_spr, SpriteID new_spr)
 
 	scnew->file_slot = scold->file_slot;
 	scnew->file_pos = scold->file_pos;
-	scnew->ptr = NULL;
+	scnew->ptr = nullptr;
 	scnew->id = scold->id;
 	scnew->type = scold->type;
 	scnew->warned = false;
@@ -643,7 +650,7 @@ void IncreaseSpriteLRU()
 
 		for (i = 0; i != _spritecache_items; i++) {
 			SpriteCache *sc = GetSpriteCache(i);
-			if (sc->ptr != NULL) {
+			if (sc->ptr != nullptr) {
 				if (sc->lru >= 0) {
 					sc->lru = -1;
 				} else if (sc->lru != -32768) {
@@ -715,7 +722,7 @@ static void DeleteEntryFromSpriteCache(uint item)
 	MemBlock *s = (MemBlock*)GetSpriteCache(item)->ptr - 1;
 	assert(!(s->size & S_FREE_MASK));
 	s->size |= S_FREE_MASK;
-	GetSpriteCache(item)->ptr = NULL;
+	GetSpriteCache(item)->ptr = nullptr;
 
 	/* And coalesce adjacent free blocks */
 	for (s = _spritecache_ptr; s->size != 0; s = NextBlock(s)) {
@@ -737,7 +744,7 @@ static void DeleteEntryFromSpriteCache()
 	cur_lru = 0xffff;
 	for (SpriteID i = 0; i != _spritecache_items; i++) {
 		SpriteCache *sc = GetSpriteCache(i);
-		if (sc->type != ST_RECOLOUR && sc->ptr != NULL && sc->lru < cur_lru) {
+		if (sc->type != ST_RECOLOUR && sc->ptr != nullptr && sc->lru < cur_lru) {
 			cur_lru = sc->lru;
 			best = i;
 		}
@@ -807,7 +814,7 @@ static void *HandleInvalidSpriteRequest(SpriteID sprite, SpriteType requested, S
 
 	SpriteType available = sc->type;
 	if (requested == ST_FONT && available == ST_NORMAL) {
-		if (sc->ptr == NULL) sc->type = ST_FONT;
+		if (sc->ptr == nullptr) sc->type = ST_FONT;
 		return GetRawSprite(sprite, sc->type, allocator);
 	}
 
@@ -837,7 +844,7 @@ static void *HandleInvalidSpriteRequest(SpriteID sprite, SpriteType requested, S
  * If the sprite is not available or of wrong type, a fallback sprite is returned.
  * @param sprite Sprite to read.
  * @param type Expected sprite type.
- * @param allocator Allocator function to use. Set to NULL to use the usual sprite cache.
+ * @param allocator Allocator function to use. Set to nullptr to use the usual sprite cache.
  * @return Sprite raw data
  */
 void *GetRawSprite(SpriteID sprite, SpriteType type, AllocatorProc *allocator)
@@ -856,14 +863,14 @@ void *GetRawSprite(SpriteID sprite, SpriteType type, AllocatorProc *allocator)
 
 	if (sc->type != type) return HandleInvalidSpriteRequest(sprite, type, sc, allocator);
 
-	if (allocator == NULL) {
+	if (allocator == nullptr) {
 		/* Load sprite into/from spritecache */
 
 		/* Update LRU */
 		sc->lru = ++_sprite_lru_counter;
 
 		/* Load the sprite, if it is not loaded, yet */
-		if (sc->ptr == NULL) sc->ptr = ReadSprite(sc, sprite, type, AllocSprite);
+		if (sc->ptr == nullptr) sc->ptr = ReadSprite(sc, sprite, type, AllocSprite);
 
 		return sc->ptr;
 	} else {
@@ -882,7 +889,7 @@ static void GfxInitSpriteCache()
 	/* Remember 'target_size' from the previous allocation attempt, so we do not try to reach the target_size multiple times in case of failure. */
 	static uint last_alloc_attempt = 0;
 
-	if (_spritecache_ptr == NULL || (_allocated_sprite_cache_size != target_size && target_size != last_alloc_attempt)) {
+	if (_spritecache_ptr == nullptr || (_allocated_sprite_cache_size != target_size && target_size != last_alloc_attempt)) {
 		delete[] reinterpret_cast<byte *>(_spritecache_ptr);
 
 		last_alloc_attempt = target_size;
@@ -893,10 +900,10 @@ static void GfxInitSpriteCache()
 				/* Try to allocate 50% more to make sure we do not allocate almost all available. */
 				_spritecache_ptr = reinterpret_cast<MemBlock *>(new byte[_allocated_sprite_cache_size + _allocated_sprite_cache_size / 2]);
 			} catch (std::bad_alloc &) {
-				_spritecache_ptr = NULL;
+				_spritecache_ptr = nullptr;
 			}
 
-			if (_spritecache_ptr != NULL) {
+			if (_spritecache_ptr != nullptr) {
 				/* Allocation succeeded, but we wanted less. */
 				delete[] reinterpret_cast<byte *>(_spritecache_ptr);
 				_spritecache_ptr = reinterpret_cast<MemBlock *>(new byte[_allocated_sprite_cache_size]);
@@ -906,7 +913,7 @@ static void GfxInitSpriteCache()
 				/* Try again to allocate half. */
 				_allocated_sprite_cache_size >>= 1;
 			}
-		} while (_spritecache_ptr == NULL);
+		} while (_spritecache_ptr == nullptr);
 
 		if (_allocated_sprite_cache_size != target_size) {
 			DEBUG(misc, 0, "Not enough memory to allocate %d MiB of spritecache. Spritecache was reduced to %d MiB.", target_size / 1024 / 1024, _allocated_sprite_cache_size / 1024 / 1024);
@@ -931,7 +938,7 @@ void GfxInitSpriteMem()
 	/* Reset the spritecache 'pool' */
 	free(_spritecache);
 	_spritecache_items = 0;
-	_spritecache = NULL;
+	_spritecache = nullptr;
 
 	_compact_cache_counter = 0;
 }
@@ -945,7 +952,7 @@ void GfxClearSpriteCache()
 	/* Clear sprite ptr for all cached items */
 	for (uint i = 0; i != _spritecache_items; i++) {
 		SpriteCache *sc = GetSpriteCache(i);
-		if (sc->type != ST_RECOLOUR && sc->ptr != NULL) DeleteEntryFromSpriteCache(i);
+		if (sc->type != ST_RECOLOUR && sc->ptr != nullptr) DeleteEntryFromSpriteCache(i);
 	}
 }
 
