@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -31,7 +29,7 @@ TownNameParams::TownNameParams(const Town *t) :
 		grfid(t->townnamegrfid), // by default, use supplied data
 		type(t->townnametype)
 {
-	if (t->townnamegrfid != 0 && GetGRFTownName(t->townnamegrfid) == NULL) {
+	if (t->townnamegrfid != 0 && GetGRFTownName(t->townnamegrfid) == nullptr) {
 		/* Fallback to english original */
 		this->grfid = 0;
 		this->type = SPECSTR_TOWNNAME_ENGLISH;
@@ -92,16 +90,15 @@ bool VerifyTownName(uint32 r, const TownNameParams *par, TownNames *town_names)
 	/* Check size and width */
 	if (Utf8StringLength(buf1) >= MAX_LENGTH_TOWN_NAME_CHARS) return false;
 
-	if (town_names != NULL) {
+	if (town_names != nullptr) {
 		if (town_names->find(buf1) != town_names->end()) return false;
 		town_names->insert(buf1);
 	} else {
-		const Town *t;
-		FOR_ALL_TOWNS(t) {
+		for (const Town *t : Town::Iterate()) {
 			/* We can't just compare the numbers since
 			 * several numbers may map to a single name. */
 			const char *buf = t->name;
-			if (buf == NULL) {
+			if (buf == nullptr) {
 				GetTownName(buf2, t, lastof(buf2));
 				buf = buf2;
 			}
@@ -124,7 +121,7 @@ bool GenerateTownName(uint32 *townnameparts, TownNames *town_names)
 	TownNameParams par(_settings_game.game_creation.town_name);
 
 	/* This function is called very often without entering the gameloop
-	 * inbetween. So reset layout cache to prevent it from growing too big. */
+	 * in between. So reset layout cache to prevent it from growing too big. */
 	Layouter::ReduceLineCache();
 
 	/* Do not set i too low, since when we run out of names, we loop
@@ -504,8 +501,8 @@ static char *MakeFinnishTownName(char *buf, const char *last, uint32 seed)
 		char *end = buf - 1;
 		assert(end >= orig);
 		if (*end == 'i') *end = 'e';
-		if (strstr(orig, "a") != NULL || strstr(orig, "o") != NULL || strstr(orig, "u") != NULL ||
-				strstr(orig, "A") != NULL || strstr(orig, "O") != NULL || strstr(orig, "U")  != NULL) {
+		if (strstr(orig, "a") != nullptr || strstr(orig, "o") != nullptr || strstr(orig, "u") != nullptr ||
+				strstr(orig, "A") != nullptr || strstr(orig, "O") != nullptr || strstr(orig, "U")  != nullptr) {
 			buf = strecpy(buf, "la", last);
 		} else {
 			buf = strecpy(buf, "l\xC3\xA4", last);
