@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -234,13 +232,13 @@ static void setupApplication()
 }
 
 
-static int CDECL ModeSorter(const OTTD_Point *p1, const OTTD_Point *p2)
+static bool ModeSorter(const OTTD_Point &p1, const OTTD_Point &p2)
 {
-	if (p1->x < p2->x) return -1;
-	if (p1->x > p2->x) return +1;
-	if (p1->y < p2->y) return -1;
-	if (p1->y > p2->y) return +1;
-	return 0;
+	if (p1.x < p2.x) return true;
+	if (p1.x > p2.x) return false;
+	if (p1.y < p2.y) return true;
+	if (p1.y > p2.y) return false;
+	return false;
 }
 
 static void QZ_GetDisplayModeInfo(CFArrayRef modes, CFIndex i, int &bpp, uint16 &width, uint16 &height)
@@ -258,11 +256,10 @@ static void QZ_GetDisplayModeInfo(CFArrayRef modes, CFIndex i, int &bpp, uint16
 
 #if (MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_11)
 		/* Extract bit depth from mode string. */
-		CFStringRef pixEnc = CGDisplayModeCopyPixelEncoding(mode);
-		if (CFStringCompare(pixEnc, CFSTR(IO32BitDirectPixels), kCFCompareCaseInsensitive) == kCFCompareEqualTo) bpp = 32;
-		if (CFStringCompare(pixEnc, CFSTR(IO16BitDirectPixels), kCFCompareCaseInsensitive) == kCFCompareEqualTo) bpp = 16;
-		if (CFStringCompare(pixEnc, CFSTR(IO8BitIndexedPixels), kCFCompareCaseInsensitive) == kCFCompareEqualTo) bpp = 8;
-		CFRelease(pixEnc);
+		CFAutoRelease<CFStringRef> pixEnc(CGDisplayModeCopyPixelEncoding(mode));
+		if (CFStringCompare(pixEnc.get(), CFSTR(IO32BitDirectPixels), kCFCompareCaseInsensitive) == kCFCompareEqualTo) bpp = 32;
+		if (CFStringCompare(pixEnc.get(), CFSTR(IO16BitDirectPixels), kCFCompareCaseInsensitive) == kCFCompareEqualTo) bpp = 16;
+		if (CFStringCompare(pixEnc.get(), CFSTR(IO8BitIndexedPixels), kCFCompareCaseInsensitive) == kCFCompareEqualTo) bpp = 8;
 #else
 		/* CGDisplayModeCopyPixelEncoding is deprecated on OSX 10.11+, but there are no 8 bpp modes anyway... */
 		bpp = 32;
@@ -326,7 +323,7 @@ uint QZ_ListModes(OTTD_Point *modes, uint max_modes, CGDirectDisplayID display_i
 	}
 
 	/* Sort list smallest to largest */
-	QSortT(modes, count, &ModeSorter);
+	std::sort(modes, modes + count, ModeSorter);
 
 #if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6)
 	if (MacOSVersionIsAtLeast(10, 6, 0)) CFRelease(mode_list);
@@ -363,12 +360,10 @@ static void QZ_UpdateVideoModes()
 	OTTD_Point modes[32];
 	uint count = _cocoa_subdriver->ListModes(modes, lengthof(modes));
 
+	_resolutions.clear();
 	for (uint i = 0; i < count; i++) {
-		_resolutions[i].width  = modes[i].x;
-		_resolutions[i].height = modes[i].y;
+		_resolutions.emplace_back(modes[i].x, modes[i].y);
 	}
-
-	_num_resolutions = count;
 }
 
 /**
@@ -689,15 +684,13 @@ void CocoaDialog(const char *title, const char *message, const char *buttonLabel
 void cocoaSetApplicationBundleDir()
 {
 	char tmp[MAXPATHLEN];
-	CFURLRef url = CFBundleCopyResourcesDirectoryURL(CFBundleGetMainBundle());
-	if (CFURLGetFileSystemRepresentation(url, true, (unsigned char*)tmp, MAXPATHLEN)) {
+	CFAutoRelease<CFURLRef> url(CFBundleCopyResourcesDirectoryURL(CFBundleGetMainBundle()));
+	if (CFURLGetFileSystemRepresentation(url.get(), true, (unsigned char*)tmp, MAXPATHLEN)) {
 		AppendPathSeparator(tmp, lastof(tmp));
 		_searchpaths[SP_APPLICATION_BUNDLE_DIR] = stredup(tmp);
 	} else {
 		_searchpaths[SP_APPLICATION_BUNDLE_DIR] = NULL;
 	}
-
-	CFRelease(url);
 }
 
 /**
